/* Copyright (c) 2013 Asustor Inc. All rights reserved. */

Ext.define('AS.ARC.apps.hidriveBackup.DateSelector', {
	extend: 'Ext.form.field.Date',
	alias: 'widget.appHidriveDate',

	constructor: function(config) {
		var me = this;

		config = config || {};

		if (config.date_format) {
			config.format = me.getDateFormat(config.date_format);
		}

		Ext.apply(me, config);

		me.callParent();
	},

	setDateFormat: function(dateFormat) {
		var me = this;
		var tmp = me.getValue();

		me.format = me.getDateFormat(dateFormat);

		me.setValue(tmp);
	},

	getDateFormat: function(dateFormat) {
		var me = this;

		if (!AS.ARC.datas.dateFormat[dateFormat]) {
			return AS.ARC.datas.dateFormat[0].fieldFormat;

		} else {
			return AS.ARC.datas.dateFormat[dateFormat].fieldFormat;
		}

		/* 
		 * format reference from as-arc-util.js
		 *
		 
		AS.ARC.datas.dateFormat = [{
			fieldFormat: 'Y/m/d',
			displayFormat: '{0}/{1}/{2}'
		}, {
			fieldFormat: 'Y-m-d',
			displayFormat: '{0}-{1}-{2}'
		}, {
			fieldFormat: 'Y.m.d',
			displayFormat: '{0}.{1}.{2}'
		}, {
			fieldFormat: 'm/d/Y',
			displayFormat: '{1}/{2}/{0}'
		}, {
			fieldFormat: 'm-d-Y',
			displayFormat: '{1}-{2}-{0}'
		}, {
			fieldFormat: 'm.d.Y',
			displayFormat: '{1}.{2}.{0}'
		}, {
			fieldFormat: 'd/m/Y',
			displayFormat: '{2}/{1}/{0}'
		}, {
			fieldFormat: 'd-m-Y',
			displayFormat: '{2}-{1}-{0}'
		}, {
			fieldFormat: 'd.m.Y',
			displayFormat: '{2}.{1}.{0}'
		}];
		
		*/
	}

});

Ext.define('AS.ARC.apps.hidriveBackup.TimeSelector', {
	extend: 'Ext.container.Container',
	alias: 'widget.appHidriveTime',

	layout: 'hbox',
	itemId: 'time',

	initComponent: function() {
		var me = this;

		var storeHour = Ext.create('Ext.data.ArrayStore', {
			fields: ['display', 'value'],
			data: AS.ARC.datas.getHour12()
		});

		var storeMinute = Ext.create('Ext.data.ArrayStore', {
			fields: ['display', 'value'],
			data: AS.ARC.datas.getMinute()
		});

		var storeAmPm = Ext.create('Ext.data.ArrayStore', {
			fields: ['display', 'value'],
			data: [
				[_S('COMMON', 'AM'), 'am'],
				[_S('COMMON', 'PM'), 'pm']
			]
		});

		Ext.apply(me, {
			items: [{
				xtype: 'displayfield',
				itemId: 'time-label',
				fieldLabel: AS.layout.getIndent(1, _S('COMMON', 'TIME')),
				labelWidth: AS.ARC.config.layout.defaultLabelWidth + (AS.ARC.config.layout.indentWidth * 1)
			}, {
				xtype: 'combo',
				itemId: 'am-pm',
				width: 77,
				margin: '0 5 0 0',
				store: storeAmPm,
				displayField: 'display',
				valueField: 'value',
				queryMode: 'local',
				triggerAction: 'all',
				editable: false,
				value: 'am'
			}, {
				xtype: 'combo',
				itemId: 'hour',
				width: 70,
				store: storeHour,
				displayField: 'display',
				valueField: 'value',
				queryMode: 'local',
				triggerAction: 'all',
				editable: false,
				value: '1'
			}, {
				xtype: 'displayfield',
				style: { margin: '0 1' },
				value: '&nbsp;:&nbsp;'
			}, {
				xtype: 'combo',
				itemId: 'minute',
				width: 70,
				store: storeMinute,
				displayField: 'display',
				valueField: 'value',
				queryMode: 'local',
				triggerAction: 'all',
				editable: false,
				value: '0'
			}]
		});

		me.callParent();
	},

	getTimeFormat: function() {
		var me = this,
			comboAmPm = me.items.get('am-pm');

		return (comboAmPm.isVisible() ? 0 : 1);
	},

	// time format
	// 0: 12-hour clock
	// 1: 24-hour clock
	setTimeFormat: function(format) {
		var me = this;

		if (me.getTimeFormat() === format) { return; }

		var allMinutes = me.getTime(),
			hour = Math.floor(allMinutes / 60),
			minute = allMinutes % 60;

		me.setTime({
			format: format,
			hour: hour,
			minute: minute
		});
	},

	// return minute of the day
	// type: return type, 'hour' or 'minutes', default is 'minute' 
	getTime: function(type) {
		var me = this,
			format = me.getTimeFormat(),
			type = type || 'minute';

		var comboFormat = me.items.get('am-pm'),
			comboHour = me.items.get('hour'),
			comboMinute = me.items.get('minute');

		var hour, minute;

		// 0: 12-hour clock
		// 1: 24-hour clock
		switch (format) {
			default:
			case 0:
				hour = comboHour.getValue();
				if (comboFormat.getValue() === 'am') {
					if (hour === 12) {
						hour = 0;
					}
				} else {
					if (hour !== 12) {
						hour += 12;
					}
				}

				minute = comboMinute.getValue();
				break;

			case 1:
				hour = comboHour.getValue();
				minute = comboMinute.getValue();
				break;
		}

		if (type === 'hour') {
			return {
				hour: hour,
				minute: minute
			};

		} else {
			return hour * 60 + minute;
		}
	},

	// 	config = {
	//		hour: 0-23,
	//		minute: 0-59
	//	}
	setTime: function(config) {
		var me = this,
			format = config.format || me.getTimeFormat();

		var comboAmPm = me.items.get('am-pm'),
			comboHour = me.items.get('hour'),
			comboMinute = me.items.get('minute');

		switch (format) {
			default:
			case 0:
				if (config.hour <= 11) {
					comboAmPm.setValue('am');
				} else {
					comboAmPm.setValue('pm');
				}

				if (config.hour <= 0 || config.hour >= 13) {
					config.hour = Math.abs(config.hour -= 12);
				}

				comboAmPm.show();
				comboHour.store.loadData(AS.ARC.datas.getHour12());
				comboHour.setValue(config.hour);
				break;


			case 1:
				comboAmPm.hide();
				comboHour.store.loadData(AS.ARC.datas.getHour24());
				comboHour.setValue(config.hour);
				break;

		}

		comboMinute.setValue(config.minute);
	}
});

Ext.define('AS.ARC.apps.hidriveBackup.Tree', {
	extend: 'Ext.tree.Panel',
	alias: 'widget.appHidriveTree',

	itemId: 'tree',
	rootVisible: false,
	useArrows: true,
	treeWidth: 200,
	height: 250,
	resizable: false,
	displayField: 'name',
	oneMode: false,
	timeout: 1000 * 60 * 5, // 5 min

	parentWin: null,

	apiUrl: null, // This is for store to get childs
	act: 'list', // This is the action to get childs

	tri: {
		NONE: 0,
		HALF: 1,
		ALL: 2
	},

	initComponent: function() {
		var me = this;

		me.addEvents({
			exception: true
		});

		me.setDefaultStore();

		me.callParent();
	},

	setDefaultStore: function() {
		var me = this;

		me.store = Ext.create('Ext.data.TreeStore', {
			autoLoad: false,
			autoDestroy: true,
			fields: [
				{ name: 'id', mapping: 'path' },
				{ name: 'name' },
				{ name: 'path' },
				{ name: 'check' }
			],
			nodeParam: 'path',
			proxy: {
				type: 'ajax',
				url: AS.ARC.util.getApiUrlWithSid(me.apiUrl, { act: me.act }),
				method: 'post',
				timeout: me.timeout,
				actionMethods: {
					create: 'POST',
					read: 'POST',
					update: 'POST',
					destroy: 'POST'
				},
				reader: {
					type: 'json',
					root: 'folders'
				},
				listeners: {
					exception: function(proxy, response, operation, opts) {
						me.fireEvent('exception', proxy, response, operation, opts);

						// if (operation.error && operation.error.status === -1) {return;}

						// if (Ext.JSON.decode(response.responseText)) { // not aborted
						// AS.ARC.util.showMsgWindow({
						// 5001: _S('ERROR', 'PRIVATEKEY_ERROR'),
						// 5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
						// 5025: _S('ERROR', 'NO_OBTAIN_LIST_RIGHTS'),
						// 5028: _S('HIDRIVE_BACKUP', 'JOB_NOT_EXIST'),
						// 5801: _S('ERROR', 'CONNECTION_FIALED')
						// }, Ext.JSON.decode(response.responseText), me.up('window'), 'warning', function () {
						// operation.node.collapse();
						// });
						// }
					}
				}
			},
			// to set the status of the node which is checkable
			setTriStatus: function(node, status) {
				if (node.isRoot()) {
					return;
				}

				switch (status) {
					default:
					case me.tri.NONE:
						node.set('checked', false);
						node.set('tri', me.tri.NONE);
						node.set('cls', '');
						break;
					case me.tri.HALF:
						node.set('checked', false);
						node.set('tri', me.tri.HALF);
						node.set('cls', 'x-tree-checked-down');
						break;
					case me.tri.ALL:
						node.set('checked', true);
						node.set('tri', me.tri.ALL);
						node.set('cls', '');
						break;
				}
			},
			listeners: {
				beforeload: function(store, operation) {
					store.lastOperation = operation;

					if (me.el && operation.node.isRoot()) {
						me.el.mask(_S('COMMON', 'LOADING') + '...');
					}

					if (operation.node.isRoot()) {
						operation.params.path = '/';
					}

					if (me.extraParam.id) {
						operation.params.id = me.extraParam.id;
					} else {
						operation.params.username = me.extraParam.username;
						operation.params.password = me.extraParam.password;
						operation.params.protocol = me.extraParam.protocol;
					}
				},
				load: function(store, node) {
					if (me.parentWin) me.parentWin.el.unmask();
					me.el.unmask();
				},
				beforeappend: function(parent, node, eOpts) {

				},
				append: function(parent, node) {
					if (node.get('path') == '') { return; }

					if (me.oneMode) {

					} else {
						if (node.parentNode.data.checked) {
							me.store.setTriStatus(node, me.tri.ALL);

						}
						// else if (node.parentNode.data.check === me.tri.HALF) {
						// me.el.mask(_S('COMMON', 'LOADING') + '...');

						// } 
						else { //if (node.parentNode.data.check === me.tri.NONE) {
							me.store.setTriStatus(node, node.data.check);
						}
					}
				}
			}
		});

	},

	checkParent: function(node) {
		var me = this,
			count = 0,
			half = false,
			store = me.store;

		node.bubble(function(parent) {
			if (parent.data.path === node.data.path)
				return true; // continue

			parent.eachChild(function(child) {
				if (child.data.tri > 0) {
					if (child.data.tri === me.tri.HALF) {
						half = true;
					}
					count++;
				}
			});

			if (count === parent.childNodes.length) {
				if (half) {
					store.setTriStatus(parent, me.tri.HALF);
				} else {
					//store.setTriStatus(parent, me.tri.ALL);
				}
			} else if (count > 0) {
				store.setTriStatus(parent, me.tri.HALF);

			} else {
				store.setTriStatus(parent, me.tri.NONE);
			}

			count = 0;
			half = false;
		});
	},

	checkChild: function(node) {
		var me = this;

		node.cascadeBy(function(child) {
			if (child.data.path === node.data.path)
				return true; // continue

			child.set('checked', node.data.checked);
			child.set('tri', (node.data.checked) ? me.tri.ALL : me.tri.NONE);
			child.set('cls', '');
		});
	},

	checkAllNode: function(node) {
		this.checkParent(node);
		this.checkChild(node);
	},

	// to get the paths if the node is HALF and do not has children. I know it is strange... 
	getHalfPathWithoutChild: function() {
		var me = this;

		var path = [];

		me.getRootNode().cascadeBy(function(child) {
			if (child.isRoot())
				return true; // continue

			if (!child.hasChildNodes() && child.data.tri === me.tri.HALF) {
				path.push(child.data.path);
			}
		});

		return path;
	},

	cancelRequest: function() {
		var me = this,
			store = me.store;

		if (store.loading && store.lastOperation) {
			var requests = Ext.Ajax.requests;

			for (id in requests) {
				if (requests.hasOwnProperty(id) && requests[id].options == store.lastOperation.request) {
					Ext.Ajax.abort(requests[id]);
				}
			}
		}
	},

	getSelection: function() {
		var me = this;

		if (me.oneMode) {
			return me.getSelectionModel().getSelection();
		} else {
			var selections = [];

			Ext.each(me.getChecked(), function(record) {
				if (!record.parentNode.data.checked) {
					selections.push(record);
				}
			});

			return selections;
		}
	},

	getPath: function() {
		var me = this,
			path = [];

		Ext.each(me.getSelection(), function(rec) {
			path.push(rec.data.path);
		});

		return path;
	},

	onViewReady: function() {
		var me = this;

		me.el.mask(_S('COMMON', 'LOADING') + '...');

		// overwrite checkchange
		me.getView().onCheckChange = function(record) {
			var checked = record.get('checked');
			if (Ext.isBoolean(checked)) {
				checked = !checked;
				record.set('checked', checked);

				//--
				var store = me.store;
				if (!me.oneMode) {
					if (checked) {
						store.setTriStatus(record, me.tri.ALL);
					} else {
						store.setTriStatus(record, me.tri.NONE);
					}

					me.checkAllNode(record);
				}
				//--

				this.fireEvent('checkchange', record, checked);
			}
		};

		me.callParent();
	}

});

Ext.define('AS.ARC.apps.hidriveBackup.PathSelector', {
	extend: 'Ext.container.Container',
	alias: 'widget.appHidrivePath',

	parentWin: null, // to mask on showing window

	apiUrl: null,

	act: 'list',

	createApiUrl: null,

	createAct: 'create',

	getParentWin: Ext.emptyFn, // for get parent win

	allowAdd: Ext.emptyFn, // for get parent win

	initComponent: function() {
		var me = this;

		Ext.apply(me, {
			layout: 'hbox',
			defaults: {
				//msgTarget: AS.ARC.config.msgTarget,
				blankText: _S('MESSAGE', 'HF00011')
			},
			items: [{
				xtype: 'textfield',
				itemId: 'path',
				allowBlank: false,
				readOnly: true,
				flex: true,
				value: me.value,
				padding: 0,
				margin: 0
			}, {
				xtype: 'button',
				text: _S('COMMON', 'BROWSE'),
				itemId: 'button',
				margin: '0 0 0 5',
				handler: me.getSelectWindow,
				scope: me
			}]
		})

		me.callParent();
	},

	getSelectWindow: function() {
		var me = this;

		var path = me.getValue();

		var buttonApply = Ext.widget('button', {
			text: _S('COMMON', 'OK'),
			disabled: true,
			handler: function() {
				var selections = me.getTree().getSelection();

				if (selections.length > 0) {
					me.getPathCmp().setValue(selections[0].data.path);
				}

				me.pathWin.close();
				me.pathWin = null;
			}
		});

		me.pathWin = Ext.create('AS.ARC.subWindow', {
			title: _S('COMMON', 'DIRECTORY'),
			width: 500,
			height: 425,
			layout: 'fit',
			parentWin: me.getParentWin(),
			fbar: [
				buttonApply,
				{
					text: _S('COMMON', 'CANCEL'),
					handler: function() {
						me.getTree().cancelRequest();
						me.pathWin.close();
						me.pathWin = null;
					}
				}],
			items: [{
				xtype: 'asPanelTop10',
				itemId: 'panel',
				buttonApply: buttonApply,
				items: [{
					xtype: 'button',
					text: _S('COMMON', 'CREATE_FOLDER'),
					hidden: false,
					disabled: true,
					handler: me.showAddFolder,
					scope: me
				}, {
					xtype: 'appHidriveTree',
					oneMode: true,
					animate: false,
					apiUrl: me.apiUrl,
					act: me.act,
					extraParam: me.extraParam,
					oneMode: true,
					listeners: {
						selectionchange: function(tree, record) {
							me.getTree().prev().enable();
							buttonApply.enable();
						},
						exception: function(proxy, response, operation, opts) {
							if (operation.error && operation.error.status === -1) { return; }
							var json = Ext.JSON.decode(response.responseText);
							if (json) { // not aborted
								AS.ARC.util.showMsgWindow({
									5001: _S('ERROR', 'PRIVATEKEY_ERROR'),
									5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
									5025: _S('ERROR', 'NO_OBTAIN_LIST_RIGHTS'),
									5028: _S('HIDRIVE_BACKUP', 'JOB_NOT_EXIST'),
									5039: _S('ERROR', 'CONNECTION_FIALED'),
									5801: _S('ERROR', 'CONNECTION_FIALED')
								}, json, me.pathWin, 'warning', function() {
									operation.node.collapse();
									if (json.error_code == 5001) {
										if (me.pathWin.parentWin.itemId === 'edit') {
											me.pathWin.parentWin.items.getAt(0).setActiveTab(0);
											me.pathWin.close();
										}
									}
								});
							}
						}
					}
				}]
			}]
		}).show();
	},

	getTree: function() {
		return this.pathWin.items.get('panel').items.get('tree');
	},

	getPathCmp: function() {
		return this.items.get('path');
	},

	getValue: function() {
		return this.getPathCmp().getValue();
	},

	getOriValue: function() {
		return this.getPathCmp().originalValue;
	},

	setValue: function(v) {
		this.getPathCmp().setValue(v);
	},

	setOriValue: function(v) {
		this.getPathCmp().originalValue = v;
	},

	isDirty: function() {
		return this.getPathCmp().isDirty();
	},

	showAddFolder: function() {
		var me = this;

		var selection = me.pathWin.items.get('panel').items.get('tree').getSelection();

		// if (!selection[0]) {
		// return;
		// }

		var buttonApply = Ext.widget('button', {
			text: _S('COMMON', 'OK'),
			itemId: 'apply',
			handler: me.addFolder,
			scope: me
		});

		me.folderWin = Ext.create('AS.ARC.subWindow', {
			title: _S('COMMON', 'CREATE_FOLDER'),
			width: 400,
			height: 160,
			parentWin: me.pathWin,
			layout: 'fit',
			items: {
				xtype: 'asPanelTop20',
				itemId: 'panel',
				buttonApply: buttonApply,
				defaults: {
					labelWidth: AS.ARC.config.layout.defaultLabelWidth,
					msgTarget: AS.ARC.config.msgTarget,
					blankText: _S('MESSAGE', 'HF00011')
				},
				items: [{
					xtype: 'textfield',
					itemId: 'name',
					fieldLabel: _S('COMMON', 'NEW_NAME'),
					vtype: 'createFolder',
					allowBlank: false
				}]
			},
			fbar: [
				buttonApply,
				{
					xtype: 'button',
					text: _S('COMMON', 'CANCEL'),
					handler: function() {
						me.folderWin.close();
						me.folderWin = null;
					}
				}]

		}).show();
	},

	addFolder: function(name) {
		var me = this;

		var tree = me.pathWin.items.get('panel').items.get('tree'),
			selection = tree.getSelection();

		if (!selection[0]) {
			return;
		}

		me.folderWin.el.mask(_S('COMMON', 'APPLYING') + '...');

		var folderName = me.folderWin.items.get('panel').items.get('name').getValue();

		var params = (me.createAct === 'create') ? {
			path: selection[0].data.path + '/' + folderName
		} : {
				dest_folder: folderName,
				dest_path: selection[0].data.path
			};

		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(me.createApiUrl, { act: me.createAct }),
			method: 'post',
			params: Ext.apply(me.extraParam, params),
			success: function(json, opts) {
				if (selection[0].isLoaded()) {
					selection[0].expand(false, function() {
						var child = selection[0].appendChild({
							name: folderName,
							path: selection[0].data.path + '/' + folderName,
							id: selection[0].data.path + '/' + folderName,
							tri: 0
						});

						tree.getSelectionModel().select(child);
					});
				} else {
					selection[0].expand(false, function() {
						tree.getSelectionModel().select(selection[0].findChild('name', folderName));
					});
				}

				me.folderWin.close();
			},
			failure: function(json, opts) {
				me.folderWin.el.unmask();
				AS.ARC.util.showMsgWindow({
					5001: _S('ERROR', 'PRIVATEKEY_ERROR'),
					5008: _S('MESSAGE', 'HN00007'),
					5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
					5028: _S('HIDRIVE_BACKUP', 'JOB_NOT_EXIST'),
					5039: _S('ERROR', 'CONNECTION_FIALED'),
					5801: _S('ERROR', 'CONNECTION_FIALED'),
					5008: _S('MESSAGE', 'HN00007'),
					5011: _S('ERROR', 'UNABLE_MODIFY_FILE_PROPERTIES'),
					5019: _S('ERROR', 'UNABLE_CREATE_FOLDER'),
					5025: _S('ERROR', 'UNABLE_CREATE_FOLDER_ACCESS_RIGHT'),
					5026: _S('ERROR', 'DESTINATION_FOLDER_READ_ONLY'),
					5045: _S('ERROR', 'MAXIMUN_USAGE_QUOTA'),
					5605: _S('ERROR', 'NO_SPACE_DISK')
				}, json, me.folderWin);
			}
		});
	}
});

/**
 * Display message in the toolbar for tip
 */
Ext.define('AS.ARC.apps.hidriveBackup.RemoteTextField', {
	extend: 'Ext.form.field.Text',
	alias: 'widget.appHidriveRemoteTextfield',

	url: null, // Remote API URL, required
	getRemoteParams: Ext.emptyFn, // Abstract method to get remote parameter, required
	remoteSuccess: Ext.emptyFn, // Abstract callback method if the request is success
	remoteFailure: Ext.emptyFn, // Abstract callback method if the request is failure
	remoteErrorText: _S('COMMON', 'INVALID_VALUE'), // Error message

	remoteValid: false,

	enableKeyEvents: true,
	checkChangeBuffer: 300, // input buffer time, 300 milisecond to check change

	value: '',

	initComponent: function() {
		var me = this;

		if (!me.validator) {
			me.validator = function() {
				if (me.getValue() && Ext.String.trim(me.getValue()).toLowerCase() !== Ext.String.trim(me.originalValue).toLowerCase()) {
					return me.remoteValid || me.remoteErrorText;
				} else {
					return true;
				}
			};
		}

		me.callParent();
	},

	// overwrite method checkChange
	checkChange: function() {
		if (!this.suspendCheckChange) {
			var me = this,
				newVal = me.getValue(),
				oldVal = me.lastValue;

			if (!me.isEqual(newVal, oldVal) && !me.isDestroyed) {
				me.lastValue = newVal;

				// remote check if valid
				me.isRemoteValid();

				me.fireEvent('change', me, newVal, oldVal);
				me.onChange(newVal, oldVal);
			}
		}
	},

	isRemoteValid: function() {
		var me = this;

		if (!me.getValue() || Ext.String.trim(me.getValue()).toLowerCase() === Ext.String.trim(me.originalValue).toLowerCase()) { return; }

		me.remoteErrorText = _S('COMMON', 'LOADING') + '...';
		me.setRemoteValid(false);

		var params = me.getRemoteParams(me);

		AS.ARC.ajax({
			url: me.url,
			params: params,
			method: 'POST',
			success: function(json) {
				if (me.getValue() === params.name) {
					me.remoteSuccess(me, json);
				}
			},
			failure: function(r) {
				if (me.getValue() === params.name) {
					me.remoteFailure(me, r);
					me.setRemoteValid(false);
				}
			}
		});
	},

	setRemoteValid: function(valid) {
		var me = this;

		if (valid) {
			me.clearInvalid();
		}

		me.remoteValid = valid;
		me.isValid();
	}
});

/**
 * @class AS.ARC.apps.hidriveBackup.core
 * @extends Ext.util.Observable
 * <p>core</p>
 */
Ext.define('AS.ARC.apps.hidriveBackup.core', {
	extend: 'Ext.util.Observable',

	constructor: function(config) {
		var fn = this;

		Ext.apply(fn, config);

		fn.callParent();

		fn.init();
	},

	init: function() {
		var fn = this;

		fn.API = {
			task: fn.getApiUrl('task.cgi'),
			remote: fn.getApiUrl('remote.cgi'),
			local: fn.getApiUrl('local.cgi'),
			regional: AS.ARC.util.getApiUrl('regional.cgi', 'settings'),
			fileExplorer: AS.ARC.util.getApiUrl('fileExplorer.cgi', 'fileExplorer')
		};

		fn.image = {
			bgLeft: fn.getImageUrl('bg-left.png'),
			bgMain: fn.getImageUrl('bg-main.png'),
			info: fn.getImageUrl('info.png')
		};

		fn.list = {
			stop: false,
			timeout: 3000,
			id: -1
		};

		fn.status = {
			FINISH: 0,
			FAILURE: 1,
			NEVER_EXEC: 2,
			NO_INFO: 3,
			BACKING_UP: 4,
			ABORTED: 5
		};

		fn.initFieldValidate();

		fn.renderWindow();

		fn.loadData();
	},

	getApiUrl: function(cgi) {
		return AS.ARC.util.getUserAppsPath() + this.app.appTag + '/' + cgi;
	},

	getImageUrl: function(filename) {
		return AS.ARC.util.getUserAppsPath() + this.app.appTag + '/images/' + filename;
	},

	initFieldValidate: function() {
		var fn = this;

		// Validate hostname/IP
		// Ext.form.VTypes.ralusHostnameRe = /^(([01]?[0-9]?[0-9]|2([0-4][0-9]|5[0-5]))\.){3}([01]?[0-9]?[0-9]|2([0-4][0-9]|5[0-5]))$|^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])\.)+([a-zA-Z0-9]{2,5})$/;
		// Ext.form.VTypes.ralusHostnameText = _S('RALUS_AGENT', 'HOSTNAME_IP_ERROR');
		// Ext.form.VTypes.ralusHostname = function (v) {
		// return Ext.form.VTypes.ralusHostnameRe.test(v);
		// };
	},

	renderWindow: function() {
		var fn = this;

		fn.win = fn.desktop.createWindow({
			app: fn.app,
			id: fn.id,
			itemId: fn.id,
			width: 880,
			height: 520,
			resizable: true,
			border: false,
			layout: 'border',
			bodyStyle: 'padding: 0; border: none; background: #FFFFFF;',
			items: [
				fn.renderWestPanel(),
				fn.renderMainPanel()
			],
			listeners: {
				beforeclose: function() {
					clearTimeout(fn.list.id);
				},
				afterrender: function(win) {
					win.header.items.items[1].hide();
				}
			}
		});
	},

	renderWestPanel: function() {
		var fn = this;

		fn.westPanel = Ext.create('AS.ARC.fnFormPanel', {
			cls: 'as-page-panel',
			page: fn,
			region: 'west',
			width: 200,
			height: 500,
			bodyStyle: 'border:none; background:transparent; ',
			style: 'background-image: url(' + fn.image.bgLeft + ');'
			+ 'background-repeat: no-repeat;'
			+ 'border:none;'
			+ 'background-position-y: 100%;'
		});

		return fn.westPanel;
	},

	renderMainPanel: function() {
		var fn = this;

		fn.btnAdd = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'CREATE'),
			itemId: 'add',
			handler: fn.getRegionalInfo,
			scale: AS.ARC.config.defaultActionButtonSize,
			scope: fn
		});

		fn.btnAction = Ext.create('Ext.button.Button', {
			text: _S('REMOTESYNC', 'BACKUP_NOW'),
			itemId: 'action',
			handler: function() {
				fn.handleAction();
			},
			disabled: true,
			scale: AS.ARC.config.defaultActionButtonSize,
			scope: fn
		});

		fn.btnEdit = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'EDIT'),
			itemId: 'edit',
			handler: fn.getEditData,
			disabled: true,
			scale: AS.ARC.config.defaultActionButtonSize,
			scope: fn
		});

		fn.btnRemove = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'REMOVE'),
			itemId: 'remove',
			handler: fn.handleRemove,
			disabled: true,
			scale: AS.ARC.config.defaultActionButtonSize,
			scope: fn
		});

		fn.btnInfo = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'INFO'),
			itemId: 'info',
			handler: function() { fn.getEditData('info') },
			disabled: true,
			scale: AS.ARC.config.defaultActionButtonSize,
			scope: fn
		});

		var dateRenderer = function(value, metadata) {
			var json = fn.gridMain.store.proxy.reader.jsonData;

			if (value === 0) { return '--'; }

			var date = new Date(value);

			var params = {
				year: date.getUTCFullYear(),
				month: date.getUTCMonth() + 1,
				day: date.getUTCDate(),
				hour: date.getUTCHours(),
				minute: date.getUTCMinutes()
			};

			Ext.apply(params, {
				dateFormat: json.date_format,
				timeFormat: json.time_format
			});

			return fn.getDateTimeString(params);
		};

		var getErrorMsg = function(code) {
			switch (code) {
				case 5011:
					return _S('ERROR', 'IO_ERROR');
				case 5601:
					return _S('ERROR', 'CONNECTION_FAIL');
				case 5004:
					return _S('ERROR', 'INVALID_PORT');
				case 5602:
					return _S('ERROR', 'FILE_NAME_TOO_LONG');
				case 5603:
					return _S('ERROR', 'INCORRECT_USER');
				case 5614:
					return _S('ERROR', 'INCORRECT_SSH');
				case 5604:
					return _S('ERROR', 'INCORRECT_SSL');
				case 5605:
					return _S('ERROR', 'DISK_FULL');
				case 5606:
					return _S('ERROR', 'CONNECTION_REFUSED');
				case 5607:
					return _S('ERROR', 'CONNECTION_TERMINATED');
				case 5608:
					return _S('ERROR', 'CONNECTION_TIMEOUT');
				case 5609:
					return _S('ERROR', 'NO_EXTERNAL_DEVICE');
				case 5610:
					return _S('ERROR', 'DESTINATION_READ_ONLY');
				case 5611:
					return _S('ERROR', 'NO_FOLDER');
				case 5612:
					return _S('ERROR', 'NO_FILE_ATTRIBUTE');
				case 5613:
					return _S('ERROR', 'UNABLE_ACCESS_LOCKED_FILE');
				case 5615:
					return _S('ERROR', 'UNDEFINED_ERROR_BACKUP');
				case 5617:
					return _S('ERROR', 'UNABLE_TRANSFER_FAT_LIMIT');
				case 5631:
					return _S('ERROR', 'ACL_NOT_SUPPORTED');
				case 5025:
					return _S('ERROR', 'NO_PRIVILEGE');
				default:
					return _S('ERROR', 'UNDEFINED_ERROR_BACKUP');
			}
		};

		fn.gridMain = Ext.create('Ext.grid.Panel', {
			store: fn.createMainStore(),
			multiSelect: true,
			border: 1,
			style: {
				border: '1px solid #CCC'
			},
			anchor: '100%, -80',
			columns: [{
				dataIndex: 'name',
				text: _S('COMMON', 'NAME'),
				flex: 1,
				renderer: function(value) {
					return AS.ARC.util.replaceHtmlSymbols(value);
				}
			}, {
				dataIndex: 'last',
				text: _S('REMOTESYNC', 'COLUMN_LAST_BACKUP_DATE'),
				flex: 1,
				renderer: dateRenderer
			}, {
				dataIndex: 'next',
				text: _S('REMOTESYNC', 'COLUMN_NEXT_BACKUP_DATE'),
				flex: 1,
				renderer: dateRenderer
			}, {
				dataIndex: 'protocol',
				text: _S('COMMON', 'PROTOCOL'),
				flex: 1
			}, {
				dataIndex: 'status',
				text: _S('COMMON', 'STATUS'),
				flex: 1,
				renderer: function(value, metadata, record) {
					switch (value) {
						case fn.status.FINISH:
							return _S('STATUS', 'FINISHED');

						case fn.status.FAILURE:
							var id = Ext.id();

							metadata.tdAttr = 'data-errorqtip="' + getErrorMsg(record.data.fail_code) + '"';

							Ext.defer(function() {
								Ext.get(id).on('click', function() {
									fn.showLog(record.data.id);
								});
							}, 50);
							return '<a id="' + id + '" href="#" style="color:red">' + _S('COMMON', 'FAILED') + '</a>';

						case fn.status.NEVER_EXEC:
							return '--';
						case fn.status.NO_INFO:
							return _S('HIDRIVE_BACKUP', 'CONFIG_ERROR');
						case fn.status.BACKING_UP:
							return _S('EXT_BACKUP', 'BACKING_UP');
						case fn.status.ABORTED:
							return _S('STATUS', 'ABORTED');
					}
				}
			}],
			listeners: {
				selectionchange: function(selModel, selected) {
					fn.btnRemove.disable();
					fn.btnEdit.disable();
					fn.btnInfo.disable();
					fn.btnAction.disable();

					if (selected.length > 0) {
						fn.gridMain.lastSelected = selected;

						fn.btnRemove.enable();

						if (selected.length === 1) {
							fn.btnEdit.enable();
							fn.btnInfo.enable();
						}

						var backupCount = 0;

						Ext.each(selected, function(selection) {
							if (selection.data.status === fn.status.BACKING_UP) {
								backupCount++;
							}
						});

						if (backupCount === selected.length) {
							fn.btnAction.setText(_S('DISK', 'ABORT'));
							fn.btnAction.enable();
							fn.btnEdit.disable();

						} else if (backupCount > 0) {
							fn.btnAction.disable();

						} else {
							fn.btnAction.enable();
							fn.btnAction.setText(_S('REMOTESYNC', 'BACKUP_NOW'));
						}
					}
				}
			}
		});

		fn.mainPanel = Ext.create('AS.ARC.fnFormPanel', {
			cls: 'as-page-panel',
			page: fn,
			region: 'center',
			border: false,
			bodyStyle: { background: 'none' },
			layout: { type: 'anchor' },
			items: [
				fn.renderMainTbar(),
				fn.gridMain
			]
		});

		return fn.mainPanel;
	},

	renderMainTbar: function() {
		var fn = this;

		return Ext.widget('asPanelToolbarTop', {
			defaultAlign: 'tl',
			border: false,
			itemId: 'tbar',
			items: [
				fn.btnAdd,
				fn.btnAction,
				fn.btnEdit,
				fn.btnRemove,
				fn.btnInfo
			]
		});
	},

	getDateString: function(config) {
		var fn = this;

		return Ext.String.format(AS.ARC.datas.dateFormat[config.dateFormat].displayFormat, config.year, AS.util.string.fill(config.month), AS.util.string.fill(config.day));
	},

	getTimeString: function(config) {
		var fn = this,
			result = '';

		if (!config.hour) { return result; }

		if (config.timeFormat === 0) {
			var hour = 0,
				ampm = ((config.hour - 12) >= 0) ? 'PM' : 'AM';

			if (config.hour > 12) {
				hour = config.hour - 12;
			} else if (config.hour == 0) {
				hour = 12;
			} else {
				hour = config.hour;
			}

			result += ampm + ' ' + AS.util.string.fill(hour) + ':' + AS.util.string.fill(config.minute);
		} else {
			result += AS.util.string.fill(config.hour) + ':' + AS.util.string.fill(config.minute);
		}

		return result;
	},

	getDateTimeString: function(config) {
		var fn = this;

		if (!config.year) { return '--'; }
		var result = '';

		result += fn.getDateString(config);

		result += ' ' + fn.getTimeString(config);

		return result;
	},

	showLog: function(id) {
		var fn = this;

		var panel = Ext.create('AS.ARC.panelTop10', {
			itemId: 'panel',
			border: false,
			layout: 'fit',
			items: [{
				xtype: 'textarea',
				itemId: 'error',
				readOnly: true
			}]
		});

		var win = Ext.create('AS.ARC.subWindow', {
			title: _S('SYSTEMINFORMATION', 'FN_LOG'),
			parentWin: fn.win,
			width: 700,
			height: 500,
			layout: 'fit',
			items: [
				panel
			],
			fbar: {
				items: [{
					text: _S('COMMON', 'OK'),
					handler: function() {
						win.close();
					},
					scope: this
				}]
			}
		}).show();

		win.el.mask(_S('COMMON', 'LOADING') + '...');

		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(fn.API.task, { act: 'error' }),
			method: 'post',
			params: {
				id: id
			},
			success: function(json, opts) {
				win.el.unmask();

				panel.items.get('error').setValue(json.content);
			},
			failure: function(json, opts) {
				win.el.unmask();

				AS.ARC.util.showMsgWindow({
					5302: _S('ERROR', 'INCORRECT_PARAMETER'),
					5018: _S('HIDRIVE_BACKUP', 'JOB_NOT_EXIST')
					// 5015: _S('ERROR', 'UNABLE_OBTAIN_LOG')
				}, json, win);
			}
		});
	},

	panelScheduleItems: function(isEdit, missionConfig) {
		var fn = this;

		var missionMode = new fn.missionMode;

		if (missionConfig) {
			missionMode.setValue({
				enable: missionConfig.mission_mode,
				attempt: missionConfig.retries || 3,
				period: missionConfig.retry_period || 60,
				interval: missionConfig.retry_interval || 60,
				timeout: missionConfig.timeout || 60,
				mission: (missionConfig.retry_mode === 'time_limit') ? 2 : 1
			});
		}

		return [{
			xtype: 'displayfield',
			hidden: (isEdit) ? true : false,
			value: _S('REMOTESYNC', 'SPECIFY_BACKUP_METHOD')
		}, {
			xtype: 'checkbox',
			itemId: 'now',
			hidden: (isEdit) ? true : false,
			boxLabel: _S('REMOTESYNC', 'BACKUP_NOW')
		}, {
			xtype: 'checkbox',
			itemId: 'schedule',
			boxLabel: _S('REMOTESYNC', 'SCHEDULE_BACKUP'),
			handler: function(checkbox, value) {
				var panel = checkbox.up();
				fn.onEnableSchedule(panel);
			}
		}, {
			xtype: 'combo',
			itemId: 'frequency',
			width: 423.5,
			hidden: true,
			fieldLabel: AS.layout.getIndent(1, _S('REMOTESYNC', 'BACKUP_FREQUENCY')),
			store: new Ext.data.ArrayStore({
				fields: ['display', 'value'],
				data: [
					[_S('COMMON', 'ONCE'), 'O'],
					[_S('COMMON', 'DAILY'), 'D'],
					[_S('COMMON', 'WEEKLY'), 'W'],
					[_S('COMMON', 'MONTHLY'), 'M']
				]
			}),
			displayField: 'display',
			valueField: 'value',
			queryMode: 'local',
			triggerAction: 'all',
			editable: false,
			value: 'O',
			listeners: {
				change: function(combo) {
					var panel = combo.up();
					fn.onChangeFrequency(combo.value, panel);
				}
			}
		}, {
			xtype: 'appHidriveDate',
			itemId: 'date',
			format: 'Y/m/d',
			fieldLabel: AS.layout.getIndent(1, _S('COMMON', 'DATE')),
			value: new Date(),
			editable: false,
			minValue: (isEdit) ? null : new Date(),
			hidden: true
		}, {
			xtype: 'combo',
			itemId: 'week-day',
			width: 423.5,
			hidden: true,
			multiSelect: true,
			allowBlank: false,
			fieldLabel: AS.layout.getIndent(1, _S('COMMON', 'DAY')),
			store: new Ext.data.ArrayStore({
				fields: ['display', 'value'],
				data: [
					[_S('COMMON', 'MON'), 1],
					[_S('COMMON', 'TUE'), 2],
					[_S('COMMON', 'WED'), 3],
					[_S('COMMON', 'THU'), 4],
					[_S('COMMON', 'FRI'), 5],
					[_S('COMMON', 'SAT'), 6],
					[_S('COMMON', 'SUN'), 7]
				]
			}),
			displayField: 'display',
			valueField: 'value',
			queryMode: 'local',
			triggerAction: 'all',
			editable: false,
			value: 1
		}, {
			xtype: 'combo',
			itemId: 'month-day',
			width: 423.5,
			hidden: true,
			multiSelect: true,
			allowBlank: false,
			fieldLabel: AS.layout.getIndent(1, _S('COMMON', 'DAY')),
			store: Ext.create('Ext.data.ArrayStore', {
				fields: ['display', 'value'],
				data: AS.ARC.datas.getDay()
			}),
			displayField: 'display',
			valueField: 'value',
			queryMode: 'local',
			triggerAction: 'all',
			editable: false,
			value: 1
		}, {
			xtype: 'appHidriveTime',
			hidden: true
		}, {
			xtype: 'displayfield',
			width: 'inherit',
			hidden: false,
			value: _S('BACKUP_RESTORE', 'MISSION_MODE_RETRY')
		}, {
			xtype: 'button',
			text: _S('COMMON', 'MISSIONMODE'),
			itemId: 'mission',
			width: 'inherit',
			hidden: false,
			margin: '15 0 0 0',
			missionMode: missionMode,
			handler: function(button) {
				var panel = button.up(),
					parentWin = (panel.up().xtype === 'window' ? panel.up() : panel.up().up());

				fn.handleMissionMode(button, missionMode, parentWin);
			}
		}];
	},

	panelOptionItems: function(task) {
		var fn = this;

		task = task || {
			options: {}
		};

		var nameField = (task.name) ? {
			xtype: 'displayfield',
			fieldLabel: _S('FTPBACKUP', 'JOB_NAME'),
			itemId: 'name',
			allowBlank: false,
			width: 400,
			maxLength: 32,
			maxLengthText: _S('MESSAGE', 'HF00016'),
			value: task.name,
		} : {
				xtype: 'appHidriveRemoteTextfield',
				itemId: 'name',
				width: 400,
				maxLength: 32,
				fieldLabel: _S('FTPBACKUP', 'JOB_NAME'),
				allowBlank: false,
				// vtype: 'albumName',
				remoteErrorText: _S('ERROR', 'ALBUM_EXIST'),
				getRemoteParams: function(field) {
					return {
						name: field.getValue()
					};
				},
				url: AS.ARC.util.getApiUrlWithSid(fn.API.task, { act: 'validate' }),
				remoteSuccess: function(field, json) {
					field.setRemoteValid(true);
				},
				remoteFailure: function(field, json) {
					if (json.error_code === 5008) {
						field.remoteErrorText = _S('MESSAGE', 'HN00020');
						field.setRemoteValid(false);
					} else if (json.error_code === 5041) {
						field.remoteErrorText = Ext.String.format(_S('MESSAGE', 'HF00022'), ' # $ % * + = [ ] | \\ : ; " \' < > , ? / ') + '<br>' +
							Ext.String.format(_S('MESSAGE', 'HF00021'), '@ -') + '<br>' +
							_S('MESSAGE', 'HF00020');

						field.setRemoteValid(false);
					}
				}
			};

		return [{
			xtype: 'displayfield',
			value: _S('USER', 'ENTER_INFO')
		},
			nameField,
		{
			xtype: 'combo',
			fieldLabel: _S('FTPBACKUP', 'IF_FILE_EXISTS'),
			itemId: 'exist',
			hidden: true,
			width: 400,
			store: new Ext.data.ArrayStore({
				fields: ['display', 'value'],
				data: [
					[_S('FTPBACKUP', 'OVERWRITE'), true],
					[_S('FTPBACKUP', 'SKIP'), false]
				]
			}),
			displayField: 'display',
			valueField: 'value',
			editable: false,
			value: true
		}, {
			xtype: 'checkbox',
			itemId: 'archive',
			boxLabel: _S('REMOTESYNC', 'ARCHIVE_MODE'),
			checked: task.options.archive
		}, {
			xtype: 'checkbox',
			itemId: 'compression',
			boxLabel: _S('REMOTESYNC', 'COMPRESS_DATA'),
			checked: task.options.compression
		}, {
			xtype: 'checkbox',
			itemId: 'metadata',
			boxLabel: _S('REMOTESYNC', 'KEEP_METADATA'),
			hidden: true,
			checked: false//task.options.metadata
		}, {
			xtype: 'checkbox',
			itemId: 'sparse',
			boxLabel: _S('REMOTESYNC', 'SUPPORT_FILES'),
			checked: task.options.sparse
		}, {
			xtype: 'checkbox',
			itemId: 'one',
			hidden: true,
			boxLabel: _S('REMOTESYNC', 'USE_SYNCHRONIZATION'),
			listeners: {
				// change: function (check, value) {
				// if (value) {
				// check.next().enable();
				// } else {
				// check.next().disable();
				// }
				// }
			}
		}, {
			xtype: 'checkbox',
			itemId: 'deletion',
			//style: AS.layout.getIndentCss(1),
			boxLabel: _S('REMOTESYNC', 'KEEP_DESTINATION_FILES'),
			checked: (task.options.deletion === false) ? true : ((task.options.deletion === true) ? false : '')
		}, {
			xtype: 'checkbox',
			itemId: 'partial',
			boxLabel: _S('HIDRIVE_BACKUP', 'RESUME_FILE'),
			checked: (task.options.partial === true) ? true : false
		}, {
			xtype: 'asFlexbox',
			itemId: 'bandwidth',
			// layout: 'hbox',
			checked: (task.options.bandwidth > 0) ? true : false,
			items: [{
				xtype: 'checkbox',
				itemId: 'enable',
				margin: '0 5 0 0',
				boxLabel: _S('REMOTESYNC', 'BANDWIDTH_LIMIT'),
				checked: (task.options.bandwidth) ? true : false
			}, {
				xtype: 'numberfield',
				itemId: 'value',
				width: 100,
				margin: '0 0 0 0',
				msgTarget: AS.ARC.config.msgTarget,
				minValue: 0,
				maxValue: 999999,
				value: (task.options.bandwidth ? (task.options.bandwidth / 1024) : 0)
			}, {
				xtype: 'container',
				width: 16,
				height: 16,
				margin: '0 0 0 5',
				html: '<img src=' + fn.image.info + ' data-qtip="' + _S('REMOTESYNC', 'BANDWIDTH_ZERO') + '">'
			}]
		}];
	},

	onEnableSchedule: function(panel) {
		var fn = this,
			panelItems = panel.items;

		if (panelItems.get('schedule').getValue()) {
			panelItems.get('frequency').show();
			panelItems.get('time').show();

			fn.onChangeFrequency(panelItems.get('frequency').getValue(), panel);

		} else {
			panelItems.get('frequency').hide();
			panelItems.get('date').hide();
			panelItems.get('week-day').hide();
			panelItems.get('week-day').disable();
			panelItems.get('week-day').clearInvalid();
			panelItems.get('month-day').hide();
			panelItems.get('month-day').disable();
			panelItems.get('month-day').clearInvalid();
			panelItems.get('time').hide();
		}
	},

	onChangeFrequency: function(value, panel) {
		var panelItems = panel.items;

		if (value == 'O') {
			panelItems.get('date').show();
			panelItems.get('week-day').hide();
			panelItems.get('week-day').disable();
			panelItems.get('week-day').clearInvalid();
			panelItems.get('month-day').hide();
			panelItems.get('month-day').disable();
			panelItems.get('month-day').clearInvalid();

		} else if (value == 'D') {
			panelItems.get('date').hide();
			panelItems.get('week-day').hide();
			panelItems.get('week-day').disable();
			panelItems.get('week-day').clearInvalid();
			panelItems.get('month-day').hide();
			panelItems.get('month-day').disable();
			panelItems.get('month-day').clearInvalid();

		} else if (value == 'W') {
			panelItems.get('date').hide();
			panelItems.get('week-day').show();
			panelItems.get('week-day').enable();
			panelItems.get('month-day').hide();
			panelItems.get('month-day').disable();
			panelItems.get('month-day').clearInvalid();

		} else if (value == 'M') {
			panelItems.get('date').hide();
			panelItems.get('week-day').hide();
			panelItems.get('week-day').disable();
			panelItems.get('week-day').clearInvalid();
			panelItems.get('month-day').show();
			panelItems.get('month-day').enable();
		}
		panel.form.isValid();
	},

	createMainStore: function() {
		var fn = this;

		var dateConvert = function(v) {
			if (v) {
				return Date.UTC(v.year, v.month - 1, v.day, v.hour, v.minute);
			} else {
				return 0;
			}
		};

		return Ext.create('Ext.data.JsonStore', {
			autoDestroy: true,
			fields: [
				'id',
				'name',
				{ name: 'last', mapping: 'time.last', convert: dateConvert },
				{ name: 'next', mapping: 'time.next', convert: dateConvert },
				'protocol',
				'status',
				'fail_code'
			],
			proxy: {
				type: 'ajax',
				method: 'post',
				url: AS.ARC.util.getApiUrlWithSid(fn.API.task, { act: 'list' }),
				reader: {
					type: 'json',
					root: 'tasks',
					total: 'total'
				},
				startParam: null,
				sortParam: null,
				pageParam: null,
				limitParam: null,
				listeners: {
					exception: function(proxy, response, operation, opts) {
						AS.ARC.util.showMsgWindow({
							5011: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
							5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR')
						}, Ext.JSON.decode(response.responseText), fn.win);
					}
				}
			},
			listeners: {
				load: function(store, record) {
					var sm = fn.gridMain.getSelectionModel();

					if (fn.gridMain.lastSelected) {
						var select = [];

						fn.gridMain.lastSelected.forEach(function(element) {
							store.each(function(record) {
								if (record.data.name === element.data.name) {
									select.push(record);
								}
							});
						});

						sm.select(select);
					}

					if (store.getCount() > 0 && sm.getCount() <= 0) {
						sm.select(0);
					}
				}
			}
		});
	},

	tryConnect: function(config) {
		var fn = this;

		config.win.el.mask(_S('COMMON', 'LOADING') + '...');

		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(fn.API.remote, { act: 'try' }),
			method: 'post',
			params: config.params,
			timeout: 1000 * 60 * 5,
			success: function(json, opts) {
				config.win.el.unmask();
				Ext.callback(config.cb);
			},
			failure: function(json, opts) {
				config.win.el.unmask();

				AS.ARC.util.showMsgWindow({
					5001: _S('ERROR', 'PRIVATEKEY_ERROR'),
					5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
					5028: _S('HIDRIVE_BACKUP', 'JOB_NOT_EXIST'),
					5039: _S('ERROR', 'CONNECTION_FIALED'),
					5801: _S('ERROR', 'CONNECTION_FIALED')
				}, json, config.win);
			}
		});
	},

	getRegionalInfo: function() {
		var fn = this;

		fn.win.el.mask(_S('COMMON', 'LOADING') + '...');
		fn.btnAdd.disable();

		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(fn.API.regional, { act: 'get' }),
			method: 'post',
			params: {
				tab: 'Time'
			},
			success: function(json, opts) {
				fn.getDefaultTaskName(json);

			},
			failure: function(json, opts) {
				fn.win.el.unmask();
				fn.btnAdd.enable();

				AS.ARC.util.showMsgWindow({
				}, json, fn.win);
			}
		});
	},

	getDefaultTaskName: function(config) {
		var fn = this;

		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(fn.API.task, { act: 'default' }),
			method: 'post',
			success: function(json, opts) {
				fn.win.el.unmask();

				fn.handleAdd(Ext.apply(config, json));

				fn.btnAdd.enable();
			},
			failure: function(json, opts) {
				fn.win.el.unmask();
				fn.btnAdd.enable();

				AS.ARC.util.showMsgWindow({
					5017: _S('MESSAGE', 'PM00007')
				}, json, fn.win);
			}
		});
	},

	getRemoteDest: function(config) {
		var fn = this;

		var win = config.field.getParentWin();

		win.el.mask(_S('COMMON', 'LOADING') + '...');

		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(fn.API.remote, { act: 'default' }),
			method: 'post',
			params: {
				username: config.username,
				password: config.password,
				protocol: config.protocol,
				name: config.name
			},
			success: function(json, opts) {
				config.field.setValue(json.path);
				config.field.setOriValue(json.path);
				win.el.unmask();
			},
			failure: function(json, opts) {
				win.el.unmask();
				// AS.ARC.util.showMsgWindow({
				// }, json, win);
			}
		});
	},

	delRemoteFolder: function(config) {
		var fn = this;

		var params = {
			path: config.path
		};

		if (config.username) {
			Ext.apply(params, {
				username: config.username,
				password: config.password,
				protocol: config.protocol
			});
		} else {
			Ext.apply(params, {
				name: config.name
			});
		}

		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(fn.API.remote, { act: 'delete' }),
			method: 'post',
			params: params,
			success: function(json, opts) {

			},
			failure: function(json, opts) {

			}
		});
	},

	handleAdd: function(config) {
		var fn = this;

		var checkType = function() {
			var infoItem = panelInfo.items,
				type = panelMode.items.get('type').getValue();

			if (type === 'RSYNC' || type === 'RSYNC+SSH') {
				//infoItem.get('exist').hide();
				infoItem.get('archive').show();
				infoItem.get('compression').show();
				// infoItem.get('metadata').show();
				infoItem.get('sparse').show();
				//infoItem.get('one').show();
				infoItem.get('deletion').show();
			} else {
				//infoItem.get('exist').show();
				infoItem.get('archive').hide();
				infoItem.get('compression').hide();
				// infoItem.get('metadata').hide();
				infoItem.get('sparse').hide();
				//infoItem.get('one').hide();
				infoItem.get('deletion').show();
			}
		};

		var panelMode = Ext.create('AS.ARC.wizard.formPanel', {
			itemId: 'panelMode',
			nextId: 'panelInfo',
			defaults: {
				labelWidth: AS.ARC.config.layout.defaultLabelWidth,
				msgTarget: AS.ARC.config.msgTarget,
				blankText: _S('MESSAGE', 'HF00011')
			},
			items: [{
				xtype: 'displayfield',
				value: _S('REMOTESYNC', 'SELECT_SERVER_TYPE_AND_TRANSFER_METHOD')
			}, {
				xtype: 'combo',
				fieldLabel: _S('REMOTESYNC', 'TRANSFER_MODE'),
				itemId: 'mode',
				width: 400,
				store: new Ext.data.ArrayStore({
					fields: ['display', 'value'],
					data: [
						[_S('HIDRIVE_BACKUP', 'NAS_TO_HIDRIVE'), 'to'],
						[_S('HIDRIVE_BACKUP', 'HIDRIVE_TO_NAS'), 'from']
					]
				}),
				displayField: 'display',
				valueField: 'value',
				editable: false,
				value: 'to',
				listeners: {
					select: function(me) {
						if (panelSource.tree) {
							var tmp = panelSource.tree;
							panelSource.tree = null;
							tmp.destroy();
						}
						if (me.getValue() === 'to') {
							panelDest.items.get('dest').show().enable();
							panelDest.items.get('pathContainer').hide().disable();
						} else {
							panelDest.items.get('dest').hide().disable();
							panelDest.items.get('pathContainer').show().enable();
							panelDest.checkRemoteDest();
						}
					},
					afterrender: function(me) {
						if (me.getValue() === 'to') {
							panelDest.items.get('dest').show().enable();
							panelDest.items.get('pathContainer').hide().disable();
							panelDest.checkRemoteDest();
						} else if (me.getValue() === 'from') {
							panelDest.items.get('dest').hide().disable();
							panelDest.items.get('pathContainer').show().enable();
							panelDest.checkRemoteDest();
						}
					}
				}
			}, {
				xtype: 'combo',
				fieldLabel: _S('HIDRIVE_BACKUP', 'TRANSFER_TYPE'),
				itemId: 'type',
				width: 400,
				store: new Ext.data.ArrayStore({
					fields: ['display', 'value'],
					data: [
						['Remote Sync', 'RSYNC'],
						['FTP', 'FTP'],
						['FTPS', 'FTPS'],
						['SFTP', 'SFTP'],
						['RSYNC+SSH', 'RSYNC+SSH']
					]
				}),
				displayField: 'display',
				valueField: 'value',
				editable: false,
				value: 'RSYNC',
				listeners: {
					change: function() {
						panelMode.allowNext = false;
					}
				}
			}, {
				xtype: 'displayfield',
				value: ''
			}, {
				xtype: 'displayfield',
				value: _S('HIDRIVE_BACKUP', 'ENTER_HIDRIVE_ACC_INFO')
			}, {
				xtype: 'textfield',
				itemId: 'username',
				width: 400,
				fieldLabel: _S('COMMON', 'USERNAME'),
				allowBlank: false,
				// value: 'store-9698',
				value: '',
				listeners: {
					change: function() {
						panelMode.allowNext = false;
					}
				}
			}, {
				xtype: 'textfield',
				itemId: 'password',
				width: 400,
				inputType: 'password',
				allowBlank: false,
				fieldLabel: _S('COMMON', 'PWD'),
				// value: 'qWexo+3W',
				value: '',
				listeners: {
					change: function() {
						panelMode.allowNext = false;
					}
				}
			}],
			allowGoNext: false,
			onNext: function() {
				if (panelMode.allowNext) {
					return true;
				} else {
					fn.tryConnect({
						win: wizard,
						params: {
							username: panelMode.items.get('username').getValue(),
							password: panelMode.items.get('password').getValue(),
							protocol: panelMode.items.get('type').getValue()
						},
						cb: function() {
							panelMode.allowNext = true;
							wizard.onNext();
						}
					});
				}
				return false;
			},
			loadData: function() {

				panelMode.form.isValid();
				wizard.footerBar.next.disable();
			},
			listeners: {
				afterrender: function(panel) {
					panel.form.isValid();
					panel.items.get('username').focus(true, 10);
				},
				fieldvaliditychange: function(panel, field, valid) {
					if (wizard) {
						if (panel.form.isValid()) {
							wizard.footerBar.next.enable();
						} else {
							wizard.footerBar.next.disable();
						}
					}
				}
			}
		});

		var panelInfo = Ext.create('AS.ARC.wizard.formPanel', {
			itemId: 'panelInfo',
			nextId: 'panelSource',
			defaults: {
				labelWidth: AS.ARC.config.layout.defaultLabelWidth,
				msgTarget: AS.ARC.config.msgTarget,
				checkChangeBuffer: AS.ARC.config.keyBuffer,
				blankText: _S('MESSAGE', 'HF00011')
			},
			items: fn.panelOptionItems(),

			loadData: checkType,

			onNext: function() { return true; },

			listeners: {
				afterrender: function(panel) {
					panel.items.get('name').setValue(config.name);

					panel.addCls('hidrive-backup option');
				},
				show: function(panel) {
					panel.items.get('name').focus(true);
				}
			}
		});

		var panelSource = Ext.create('AS.ARC.wizard.formPanel', {
			itemId: 'panelSource',
			nextId: 'panelDest',
			ignoreCheckForm: true,
			tree: null,
			items: [{
				xtype: 'displayfield',
				value: _S('REMOTESYNC', 'SELECT_SOURCE_FOLDERS_BACKUP')
			}],
			loadData: function() {
				wizard.footerBar.next.disable();

				if (panelMode.items.get('mode').getValue() === 'to') {
					var tree = fn.getTreeNas('add', 'source', wizard);
				} else {

					var tree = Ext.widget('appHidriveTree', {
						animate: false,
						apiUrl: (panelMode.items.get('mode').getValue() === 'from') ? fn.API.remote : fn.API.local,
						oneMode: panelInfo.items.get('one').getValue(),
						extraParam: (panelMode.items.get('mode').getValue() === 'from') ? {
							username: panelMode.items.get('username').getValue(),
							password: panelMode.items.get('password').getValue(),
							protocol: panelMode.items.get('type').getValue()
						} : {},
						listeners: {
							checkchange: function(node, checked) {
								if (panelSource.tree.getChecked().length > 0) {
									wizard.footerBar.next.enable();
								} else {
									wizard.footerBar.next.disable();
								}
							},
							exception: function(proxy, response, operation, opts) {
								if (operation.error && operation.error.status === -1) { return; }

								if (Ext.JSON.decode(response.responseText)) { // not aborted
									AS.ARC.util.showMsgWindow({
										5001: _S('ERROR', 'PRIVATEKEY_ERROR'),
										5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
										5025: _S('ERROR', 'NO_OBTAIN_LIST_RIGHTS'),
										5028: _S('HIDRIVE_BACKUP', 'JOB_NOT_EXIST'),
										5039: _S('ERROR', 'CONNECTION_FIALED'),
										5801: _S('ERROR', 'CONNECTION_FIALED')
									}, Ext.JSON.decode(response.responseText), wizard, 'warning', function() {
										operation.node.collapse();
									});
								}
							}
						}
					});
				}


				if (!panelSource.tree) {
					panelSource.tree = panelSource.insert(1, tree
					);
				}

				panelSource.tree.extraParam = (panelMode.items.get('mode').getValue() === 'from') ? {
					username: panelMode.items.get('username').getValue(),
					password: panelMode.items.get('password').getValue(),
					protocol: panelMode.items.get('type').getValue()
				} : {};

				if (panelSource.tree.getChecked().length > 0) {
					wizard.footerBar.next.enable();
				} else {
					wizard.footerBar.next.disable();
				}
			},
			listeners: {
				show: function(panel) {

				}
			}
		});

		var sysPath = Ext.create('Ext.form.field.Text', {
			itemId: 'dest',
			allowBlank: false,
			blankText: _S('MESSAGE', 'HF00011'),
			readOnly: true,
			flex: true,
			editable: false,
			padding: 0,
			margin: 0
		});

		var sysBrowse = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'BROWSE'),
			margin: '0 0 0 5',
			handler: function() {

				var fs = Ext.create('AS.ARC.treeSelector', {
					hideReadOnly: true,
					parentWin: wizard,
					objectPath: sysPath
				});
				fs.win.show();
			}
		});

		var pathContainer = Ext.create('Ext.container.Container', {
			itemId: 'pathContainer',
			cls: 'common-layout-horizontal-3-0',
			layout: 'hbox',
			hidden: true,
			items: [
				sysPath,
				sysBrowse
			]
		});

		var appHidrivePath = {
			xtype: 'appHidrivePath',
			itemId: 'dest',
			getParentWin: function() {
				return wizard;
			}
		};


		var panelDest = Ext.create('AS.ARC.wizard.formPanel', {
			itemId: 'panelDest',
			nextId: 'panelSchedule',
			tree: null,
			items: [{
				xtype: 'displayfield',
				value: _S('FTPBACKUP', 'SELECT_FOLDER_BACKUP')
			},
				appHidrivePath,
				pathContainer
			],
			loadData: function() {
				var nasDestPath = panelDest.items.get('pathContainer').items.get('dest'),
					hidriveDestPath = panelDest.items.get('dest');

				if (panelMode.items.get('mode').getValue() === 'to') { // NAS to hiDrive
					panelDest.items.get('dest').apiUrl = (panelMode.items.get('mode').getValue() === 'to') ? fn.API.remote : fn.API.local;
					panelDest.items.get('dest').createAct = (panelMode.items.get('mode').getValue() === 'to') ? 'create' : 'createdir';
					panelDest.items.get('dest').createApiUrl = (panelMode.items.get('mode').getValue() === 'to') ? fn.API.remote : fn.API.fileExplorer;

					panelDest.items.get('dest').extraParam = (panelMode.items.get('mode').getValue() === 'to') ? {
						username: panelMode.items.get('username').getValue(),
						password: panelMode.items.get('password').getValue(),
						protocol: panelMode.items.get('type').getValue()
					} : {};
				} else { // NAS from hiDrive
					panelDest.items.get('pathContainer').items.get('dest').apiUrl = (panelMode.items.get('mode').getValue() === 'to') ? fn.API.remote : fn.API.local;
					panelDest.items.get('pathContainer').items.get('dest').createAct = (panelMode.items.get('mode').getValue() === 'to') ? 'create' : 'createdir';
					panelDest.items.get('pathContainer').items.get('dest').createApiUrl = (panelMode.items.get('mode').getValue() === 'to') ? fn.API.remote : fn.API.fileExplorer;

					panelDest.items.get('pathContainer').items.get('dest').extraParam = (panelMode.items.get('mode').getValue() === 'to') ? {
						username: panelMode.items.get('username').getValue(),
						password: panelMode.items.get('password').getValue(),
						protocol: panelMode.items.get('type').getValue()
					} : {};
				}

				if (panelMode.items.get('mode').getValue() === 'to' && !panelDest.items.get('dest').getOriValue()) {
					panelDest.autoGetDest = true;
					fn.getRemoteDest({
						username: panelMode.items.get('username').getValue(),
						password: panelMode.items.get('password').getValue(),
						protocol: panelMode.items.get('type').getValue(),
						name: panelInfo.items.get('name').getValue(),
						field: panelDest.items.get('dest')
					});
				}
			},
			checkRemoteDest: function(checkDirty) {
				if (!panelDest.autoGetDest) { return; }

				if (panelMode.items.get('mode').getValue() === 'to') {
					if (checkDirty && !panelDest.items.get('dest').isDirty()) { return; }
				} else {
					if (checkDirty && !panelDest.items.get('pathContainer').items.get('dest').isDirty()) { return; }
				}

				if (panelMode.items.get('mode').getValue() === 'to') {
					panelDest.autoGetDest = false;
				}
			},
			onBack: function() {
				panelDest.checkRemoteDest(true);
			},
			onNext: function() {
				panelDest.checkRemoteDest(true);
				return true;
			}
		});

		var panelSchedule = Ext.create('AS.ARC.wizard.formPanel', {
			itemId: 'panelSchedule',
			nextId: 'panelPreview',
			defaults: {
				labelWidth: AS.ARC.config.layout.defaultLabelWidth + (AS.ARC.config.layout.indentWidth * 1),
				msgTarget: AS.ARC.config.msgTarget,
				width: 425,
				blankText: _S('MESSAGE', 'HF00011')
			},
			items: fn.panelScheduleItems(),
			listeners: {
				afterrender: function(panel) {
					var dateCmp = panel.items.get('date'),
						timeCmp = panel.items.get('time');

					dateCmp.setDateFormat(config.DateFormat);

					timeCmp.setTime({
						hour: config.Hour,
						minute: config.Minute
					});
					timeCmp.setTimeFormat(config.TimeFormat);
				}
			},
			onNext: function() {
				return true
			}
		});

		var gridConfirm = Ext.create('Ext.grid.Panel', {
			store: Ext.create('Ext.data.Store', {
				fields: ['name', 'value'],
				data: []
			}),
			columns: [{
				text: _S('COMMON', 'NAME'),
				flex: 1,
				sortable: false,
				dataIndex: 'name'
			}, {
				text: _S('COMMON', 'VALUE'),
				flex: 2,
				sortable: false,
				dataIndex: 'value',
				renderer: function(value, metadata) {
					if (Ext.isArray(value)) {
						var tmp = '';
						Ext.each(value, function(item, index) {
							tmp += AS.ARC.util.replaceHtmlSymbols(item);
							if (index + 1 !== value.length) {
								tmp += ',<br>';
							}
						});
						value = tmp;
					} else {
						value = AS.ARC.util.replaceHtmlSymbols(value);
					}
					metadata.tdAttr = 'data-qtip="' + AS.ARC.util.replaceHtmlSymbols(value) + '"';
					return value;
				}
			}]
		});

		var panelPreview = Ext.create('AS.ARC.wizard.formPanel', {
			itemId: 'panelPreview',
			items: [{
				xtype: 'displayfield',
				value: _S('COMMON', 'CONFIRM_FOLLOWING_SETTINGS')
			},
				gridConfirm
			],
			loadData: function() {
				var modeItems = panelMode.items,
					methodItems = panelSchedule.items,
					timeCmp = methodItems.get('time'),
					timeItems = timeCmp.items;

				var store = gridConfirm.getStore();

				var time = AS.ARC.datas.defaultValue;
				if (methodItems.get('schedule').getValue()) {
					time = methodItems.get('frequency').getDisplayValue() + ', ';

					if (methodItems.get('frequency').getValue() === 'O') {
						time += _S('COMMON', 'TIME').toLowerCase() + ': ' +
							(methodItems.get('date').getValue().getMonth() + 1) + '/' + methodItems.get('date').getValue().getDate() + '/' + methodItems.get('date').getValue().getFullYear() + ' ' +
							((timeCmp.getTimeFormat() === 0) ? (timeItems.get('am-pm').getValue() === 'am' ? 'AM ' : 'PM ') : '') +
							timeItems.get('hour').getDisplayValue() + ':' + timeItems.get('minute').getDisplayValue();
					} else {
						time += _S('COMMON', 'TIME').toLowerCase() + ': ' + (timeItems.get('am-pm').getValue() === 'am' ? 'AM' : 'PM') + ' ' +
							timeItems.get('hour').getDisplayValue() + ':' + timeItems.get('minute').getDisplayValue();
					}

					if (methodItems.get('frequency').getValue() === 'W') {
						time += ', ' + _S('COMMON', 'INTERVAL').toLowerCase() + ': ' + methodItems.get('week-day').getDisplayValue();
					} else if (methodItems.get('frequency').getValue() === 'M') {
						time += ', ' + _S('COMMON', 'INTERVAL').toLowerCase() + ': ' + methodItems.get('month-day').getDisplayValue();
					}
				}

				var data = [{
					name: AS.util.uppercaseForTitle(_S('FTPBACKUP', 'JOB_NAME')),
					value: panelInfo.items.get('name').getValue()
				}, {
					name: AS.util.uppercaseForTitle(_S('HIDRIVE_BACKUP', 'TRANSFER_TYPE')),
					value: modeItems.get('type').getDisplayValue()
				}, {
					name: AS.util.uppercaseForTitle(_S('FTPBACKUP', 'TRANSFER_MODE')),
					value: modeItems.get('mode').getDisplayValue()
				}, {
					name: AS.util.uppercaseForTitle(_S('COMMON', 'USERNAME')),
					value: modeItems.get('username').getValue()
				}, {
					name: _S('COMMON', 'TIME'),
					value: time
				}, {
					name: _S('REMOTESYNC', 'BACKUP_FOLDERS'),
					value: (modeItems.get('mode').getValue() === 'to') ? fn.getSelection(panelSource.items.get('tree-nas')).split('//') : panelSource.items.get('tree').getPath()
				}, {
					name: _S('REMOTESYNC', 'DESTINATION_FOLDER'),
					value: panelMode.items.get('mode').getValue() === 'to' ? panelDest.items.get('dest').getValue() : panelDest.items.get('pathContainer').items.get('dest').getValue()
				}];

				store.removeAll();
				store.loadData(data);

				if (gridConfirm.getHeight() > 270) {
					gridConfirm.setHeight(270)
				}
			},
			onApply: function() {
				wizard.el.mask(_S('COMMON', 'APPLYING') + '...');

				var modeItems = panelMode.items,
					infoItems = panelInfo.items,
					methodItems = panelSchedule.items,
					schedule = {};

				if (methodItems.get('schedule').getValue()) {
					timeCmp = methodItems.get('time');

					var config = {
						frequency: methodItems.get('frequency').getValue(),
						date: methodItems.get('date').getValue(),
						monthDay: methodItems.get('month-day').getValue(),
						weekDay: methodItems.get('week-day').getValue()
					};

					Ext.apply(schedule, fn.scheduleEncode(config));

					Ext.apply(schedule, timeCmp.getTime('hour'));
				}

				var missionMode = methodItems.get('mission').missionMode;

				var params = {
					name: infoItems.get('name').getValue(),
					transfer_mode: modeItems.get('mode').getValue(),
					username: modeItems.get('username').getValue(),
					password: modeItems.get('password').getValue(),
					protocol: modeItems.get('type').getValue(),
					local: (modeItems.get('mode').getValue() === 'to') ? fn.getSelection(panelSource.items.get('tree-nas'))/*panelSource.items.get('tree').getPath().join('//')*/ : panelDest.items.get('pathContainer').items.get('dest').getValue(),
					remote: (modeItems.get('mode').getValue() === 'to') ? panelDest.items.get('dest').getValue() : panelSource.items.get('tree').getPath().join('//'),
					deletion: !infoItems.get('deletion').getValue(),
					partial: infoItems.get('partial').getValue(),
					bandwidth: infoItems.get('bandwidth').items.get('enable').getValue() ? infoItems.get('bandwidth').items.get('value').getValue() * 1024 : 0,
					mission_mode: missionMode.getValue('enable'),
					retry_mode: (missionMode.getValue('mission') == 1) ? 'number_limit' : 'time_limit',
					retry_period: missionMode.getValue('period'),
					retry_interval: missionMode.getValue('interval'),
					retries: missionMode.getValue('attempt'),
					timeout: missionMode.getValue('timeout')
				};

				Ext.apply(params, schedule);

				if (params.protocol === 'RSYNC' || params.protocol === 'RSYNC+SSH') {
					Ext.apply(params, {
						archive: infoItems.get('archive').getValue(),
						compression: infoItems.get('compression').getValue(),
						metadata: infoItems.get('metadata').getValue(),
						sparse: infoItems.get('sparse').getValue()
					});
				}

				var doApply = function() {
					fn.chk_now = methodItems.get('now').getValue();
					AS.ARC.ajax({
						url: AS.ARC.util.getApiUrlWithSid(fn.API.task, { act: 'add' }),
						method: 'post',
						params: params,
						win: wizard,
						success: function(json, opts) {
							fn.gridMain.store.load(function() {
								if (fn.chk_now) {
									fn.handleBackup(json.task.id);
								}
							});
							wizard.close();
						},
						failure: function(json, opts) {
							wizard.el.unmask();

							AS.ARC.util.showMsgWindow({
								5008: _S('MESSAGE', 'HN00006'),
								5017: _S('MESSAGE', 'PM00007'),
								5011: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
								5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR')
							}, json, wizard);
						}
					});
				};

				if (modeItems.get('mode').getValue() === 'to' && panelDest.autoGetDest) {
					AS.ARC.ajax({
						url: AS.ARC.util.getApiUrlWithSid(fn.API.remote, { act: 'create' }),
						method: 'post',
						params: {
							path: panelDest.items.get('dest').getValue(),
							username: modeItems.get('username').getValue(),
							password: modeItems.get('password').getValue(),
							protocol: modeItems.get('type').getValue()
						},
						success: function(json, opts) {
							doApply();
						},
						failure: function(json, opts) {
							doApply();
							// wizard.el.unmask();
							// AS.ARC.util.showMsgWindow({
							// 5001: _S('ERROR', 'PRIVATEKEY_ERROR'),
							// 5008: _S('MESSAGE', 'HN00007'),
							// 5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
							// 5028: _S('HIDRIVE_BACKUP', 'JOB_NOT_EXIST'),
							// 5039: _S('ERROR', 'CONNECTION_FIALED'),
							// 5801: _S('ERROR', 'CONNECTION_FIALED'),
							// 5008: _S('MESSAGE', 'HN00007'),
							// 5011: _S('ERROR', 'UNABLE_MODIFY_FILE_PROPERTIES'),
							// 5019: _S('ERROR', 'UNABLE_CREATE_FOLDER'),
							// 5025: _S('ERROR', 'UNABLE_CREATE_FOLDER_ACCESS_RIGHT'),
							// 5026: _S('ERROR', 'DESTINATION_FOLDER_READ_ONLY'),
							// 5045: _S('ERROR', 'MAXIMUN_USAGE_QUOTA'),
							// 5605: _S('ERROR', 'NO_SPACE_DISK')
							// }, json, wizard);
						}
					});
				} else {
					doApply();
				}
			}
		});

		var wizard = Ext.create('AS.ARC.wizard.modalWindow', {
			title: _S('REMOTESYNC', 'ADD_NEW_BACKUP'),
			parentWin: fn.win,
			itemId: 'add',
			steps: [
				panelMode,
				panelInfo,
				panelSource,
				panelDest,
				panelSchedule,
				panelPreview
			],
			listeners: {
				beforeWizardClose: function() {
					panelDest.checkRemoteDest();
				},
				afterrender: function(win) {
					win.footerBar.next.disable();
				}
			}
		}).show();
	},

	getTreeNas: function(mode, target, wizard, btnApply) {
		var page = this,
			data = [];

		var buttonAddFolder = Ext.create('Ext.button.Button', {
			text: _S('FILE_EXPLORER', 'INFO_CREATE_FOLDER'),
			itemId: 'add',
			disabled: true,
			handler: function() {
				var sm = page.treeNas.getSelectionModel(),
					selections = sm.getSelection();

				if (selections.length < 1)
					return;

				selections[0].expand();

				var buttonApply = Ext.create('Ext.button.Button', {
					text: _S('COMMON', 'OK'),
					handler: function() {
						page.handelAddFolder(selections[0], win.items.getAt(0).items.get('name').getValue(), mode, target);
						win.close();
					}
				});

				var win = Ext.create('AS.ARC.subWindow', {
					title: _S('COMMON', 'NEW_FOLDER'),
					parentWin: (mode == 'add') ? page.wizard : page.win,
					width: 370,
					height: 160,
					layout: 'fit',
					items: [{
						xtype: 'asPanelTop20',
						buttonApply: buttonApply,
						items: [{
							xtype: 'textfield',
							fieldLabel: _S('COMMON', 'NAME'),
							itemId: 'name',
							allowBlank: false,
							blankText: _S('MESSAGE', 'HF00011'),
							msgTarget: AS.ARC.config.msgTarget
						}]
					}],
					fbar: {
						items: [
							buttonApply,
							{
								text: _S('COMMON', 'CANCEL'),
								handler: function() {
									win.close();
								},
								scope: this
							}]
					},
					listeners: {
						show: function(win) {
							win.items.items[0].items.get('name').focus(true, 10);
						}
					}
				}).show();
			}
		});
		page.treeNas = Ext.create('Ext.tree.TreePanel', {
			itemId: 'tree-nas',
			rootVisible: false,
			useArrows: true,
			//height: 230,
			treeWidth: 200,
			flex: 1,
			resizable: false,
			listeners: {
				afterrender: function(tree) {

					if (mode == 'add') {
						if (target == 'source') {
							page.treeNas.setHeight(250);
						} else { // target : 'dest'
							page.treeNas.setHeight(240);
						}
					} else { // mode : 'edit'
						if (target == 'source') {
							page.treeNas.setHeight(230);
						} else { // target : 'dest'
							page.treeNas.setHeight(230);
						}
					}

					//only not NAS as source needs create folder
					if (mode == 'add' && target == 'dest') {
						tree.ownerCt.insert(1, buttonAddFolder);
					} else if (mode == 'edit' && target == 'dest') {
						tree.ownerCt.insert(0, buttonAddFolder);
					}

					page.treeNas.setRootNode({
						text: 'NAS',
						path: '',
						expanded: true,
						tri: 0
					});

					var root = page.treeNas.getRootNode();

					if (target == 'source' && !page.sync) {
						root.set('checked', false);
					}

					var params = {
						path: "share"
					};

					// if (mode == 'edit' && target == 'source') {
					// 	Ext.apply(params, {check_box: 1, task_name: page.taskName});
					// }

					// call ajax for source tree
					AS.ARC.ajax({
						url: AS.ARC.util.getApiUrlWithSid(page.API.fileExplorer, { act: 'file_list' }),
						method: 'post',
						params: params,
						success: function(json, opts) {
							var sourcePath = page.originSourcePath;

							Ext.each(json.data, function(folder) {

								if (folder.is_dir === 0) {
									if (target == 'dest' || page.sync) {
										return true;
									}
								}

								var child = root.appendChild({
									text: folder.filename,
									path: folder.file_path,
									permission: folder.file_permission,
									dirType: folder.is_dir,
									tri: 0
								});

								if (target == 'source') {
									if (!page.sync) {
										child.set('checked', false);

										if (folder.is_dir === 0) {
											child.set('leaf', true);
										}
									}
								}
							});

							if (mode == 'edit') {

								page.expandSourceTree(root, 'nas', target);

							}
						},
						failure: function(json, opts) {
							var pWin;

							if (mode == 'add') {
								pWin = page.wizard;
							} else {
								pWin = page.win;
							}

							AS.ARC.util.showMsgWindow({
								5011: _S('ERROR', 'CANNOT_CREATE_TMP_DIRECTORY'),
								5005: _S('ERROR', 'UNABLE_OBTAIN_FILE_FOLDER_LIST'),
								5025: _S('ERROR', 'UNABLE_OBTAIN_FILE_FOLDER_LIST_ACCESS_RIGHT')
							}, json, pWin);
						}
					});
				},
				beforeitemexpand: function(node, eOpts) {
					if (!node.hasChildNodes()) {
						var params = {
							path: node.data.path
						};

						AS.ARC.ajax({
							url: AS.ARC.util.getApiUrlWithSid(page.API.fileExplorer, { act: 'file_list' }),
							method: 'post',
							params: params,
							success: function(json, opts) {
								Ext.each(json.data, function(folder) {
									var checked,
										tri,
										leaf = false;

									if (folder.permission == 3)
										return true;

									if (folder.is_dir === 0) {
										if (target == 'dest' || page.sync) {
											return false;
										} else {
											leaf = true;
										}
									}

									if (target == 'source' && !page.sync) {
										checked = node.data.checked;
										tri = (checked) ? 2 : 0;
									} else {
										checked = null;
									}
									if (folder.is_dir) {
										var child = node.appendChild({
											text: folder.filename,
											path: folder.file_path,
											permission: folder.file_permission,
											dirType: folder.is_dir,
											tri: tri,
											leaf: leaf,
											checked: checked
										});
									}

								});
							},
							failure: function(json, opts) {
								var pWin;

								if (mode == 'add') {
									pWin = page.wizard;
								} else {
									pWin = page.win;
								}

								AS.ARC.util.showMsgWindow({
									5011: _S('ERROR', 'CANNOT_CREATE_TMP_DIRECTORY'),
									5005: _S('ERROR', 'UNABLE_OBTAIN_FILE_FOLDER_LIST'),
									5025: _S('ERROR', 'UNABLE_OBTAIN_FILE_FOLDER_LIST_ACCESS_RIGHT')
								}, json, pWin);
							}
						});
					}
				},
				checkchange: function(node, checked) {
					if (target == 'dest') {
						var checkedRecord = this.getChecked();

						if (checkedRecord.length <= 0) {
							node.set('checked', true);
						} else if (checkedRecord.length > 0) {

							checkedRecord.forEach(function(record) {
								record.set('checked', (record.data.path == node.data.path) ? checked : !checked);
								record.set('cls', (record.data.path == node.data.path) ? 'x-tree-radio-selected' : 'x-tree-radio');
							});
						}

					} else { // source
						node.set('tri', (checked) ? 2 : 0);
						node.set('cls', '');
						page.checkParent(node);
						page.checkChild(node);
					}

					var checkedRecord = this.getChecked();

					if (checkedRecord.length <= 0) {
						if (mode == 'add') {
							wizard.footerBar.next.disable();
						} else {
							//console.log('ge');
							//wizard.footerBar.next.disable();
							btnApply.disable();
						}

					} else {
						if (mode == 'add') {
							wizard.footerBar.next.enable();
						} else {
							btnApply.enable();
						}
					}
				},
				selectionchange: function(sm, selections) {
					//_D(page.sync)
					if (selections.length > 0) {
						buttonAddFolder.enable();
					} else {
						buttonAddFolder.disable();
					}

					if (target == 'dest' || page.sync) {
						if (selections.length > 0) {
							if (mode == 'add') {
								wizard.footerBar.next.enable();
							} else {
								//page.win
							}
						} else {
							if (mode == 'add') {
								wizard.footerBar.next.disable();
							} else {
								//page.win
							}
						}
					}
				},
				beforeselect: function(sm, record) {
					if (target == 'dest' && (record.data.path === page.PATH_USER_HOME || record.data.module_path === page.PATH_USER_HOME)) {
						return false;
					}
				}
			}
		});

		return page.treeNas;
	},

	checkParent: function(node) {
		var count = 0, half = false;

		node.bubble(function(parent) {
			if (parent.data.path == node.data.path)
				return true; // continue

			parent.eachChild(function(child) {
				if (child.data.tri > 0) {
					if (child.data.tri == 1) {
						half = true;
					}
					count++;
				}
			});

			if (count == parent.childNodes.length) {
				if (half) {
					parent.set('checked', false);
					parent.set('tri', 1);
					parent.set('cls', 'x-tree-checked-down');
				} else {
					//parent.set('checked', true);
					parent.set('tri', 2);
					parent.set('cls', '');
				}
			} else if (count > 0) {
				parent.set('checked', false);
				parent.set('tri', 1);
				parent.set('cls', 'x-tree-checked-down');
			} else {
				parent.set('checked', false);
				parent.set('tri', 0);
				parent.set('cls', '');
			}

			count = 0;
			half = false;
		});
	},

	checkChild: function(node) {
		node.cascadeBy(function(child) {
			if (child.data.path == node.data.path)
				return true; // continue

			child.set('checked', node.data.checked);
			child.set('tri', (node.data.checked) ? 2 : 0);
			child.set('cls', '');
		});
	},

	getSelection: function(tree) {
		var selections = [];

		Ext.each(tree.getChecked(), function(record) {
			if (!record.isRoot()) {

				if (!record.parentNode.data.checked || (record.parentNode.data.checked && record.parentNode.data.path == '')) {
					selections.push(record.data.path);
				}
			}
		});
		//console.log(selections);
		return selections.join('//');
	},

	expandSourceTree: function(root, treeSource, target) {
		var page = this,
			// act = '',
			// pathes = [],
			sourcePath = page.originPath,
			total = page.originPath.length;

		// var swap = function (p, i) {
		// 	var last = p.length - 1,
		// 		temp;

		// 	temp = p[i];
		// 	p[i] = p[last];
		// 	p[last] = temp;
		// };

		if (total == 1 && page.originPath[0] == '') {
			page.finLoadSource = true;

			//page.unmaskEditWin();
		}


		var expand = function(node) {

			if (node.hasChildNodes() && node.data.cls != 'x-tree-checked-down') {   //第一次node為root, haschildnode是以父點為角度判斷是否有子節點

				node.eachChild(function(child) {  //iterate all child nodes

					sourcePath.forEach(function(path, index) {

						var parentPath = child.data.path,
							parentPathLength = parentPath.length,
							checkedPath = path,
							checkedPathLength = checkedPath.length,
							matchPath = checkedPath.indexOf(parentPath);

						if (matchPath === 0 && parentPath !== '' && parentPathLength === checkedPathLength) {
							child.set('checked', true);
						}

						if (matchPath === 0 && parentPath !== '' && parentPathLength < checkedPathLength) {
							child.set('cls', 'x-tree-checked-down');
							child.set('expanded', true);

						}

					});
					expand(child);   //遞迴 child node取代parent node

				});

			}

			if (node.data.cls == 'x-tree-checked-down') {

				var params = {
					path: node.data.path,
				};

				AS.ARC.ajax({
					url: AS.ARC.util.getApiUrlWithSid(page.API.fileExplorer, { act: 'file_list' }),
					method: 'post',
					params: params,
					success: function(json, opts) {
						// var eobranch = true;

						Ext.each(json.data, function(folder) {

							var leaf = (folder.is_dir === 0) ? true : false;

							if (folder.is_dir) {
								var child = node.appendChild({
									text: folder.filename,
									path: folder.file_path,
									permission: folder.file_permission,
									dirType: folder.is_dir,
									checked: false,
									leaf: leaf
								});
							}

						});


						var count = 0;
						node.eachChild(function(child) {
							sourcePath.forEach(function(path, index) {

								var parentPath = child.data.path,
									parentPathLength = parentPath.length,
									checkedPath = path,
									checkedPathLength = checkedPath.length,
									matchPath = checkedPath.indexOf(parentPath);

								if (matchPath === 0 && parentPath !== '' && parentPathLength === checkedPathLength) {
									child.set('checked', true);
									count++;
								}

								if (matchPath === 0 && parentPath !== '' && parentPathLength < checkedPathLength) {
									child.set('cls', 'x-tree-checked-down');
									child.set('expanded', true);

								}

								if (count == node.childNodes.length) {
									node.set('cls', '');
								}

							});

							expand(child);   //遞迴child 取代 parent node
						});

					},
					failure: function(json, opts) {
						AS.ARC.util.showMsgWindow({
							5011: _S('ERROR', 'CANNOT_CREATE_TMP_DIRECTORY'),
							5005: _S('ERROR', 'UNABLE_OBTAIN_FILE_FOLDER_LIST'),
							5025: _S('ERROR', 'UNABLE_OBTAIN_FILE_FOLDER_LIST_ACCESS_RIGHT')
						}, json, '');
					}
				});
			}

		}

		expand(root);
	},

	scheduleEncode: function(config) {
		var fn = this,
			result = {};

		if (config.frequency === 'O') {
			Ext.apply(result, {
				schedule: 'once',
				year: config.date.getFullYear(),
				month: config.date.getMonth() + 1,
				day: config.date.getDate()
			});
		} else {
			Ext.apply(result, {
				schedule: 'repeat'
			});

			if (config.frequency === 'W') {
				Ext.apply(result, {
					week: config.weekDay.join(',')
				});
			} else if (config.frequency === 'M') {
				Ext.apply(result, {
					day: config.monthDay.join(',')
				});
			}
		}

		return result;
	},

	scheduleDecode: function(config) {
		var fn = this,
			result = {};

		if (!config) { return result; }

		var period = config.period;

		if (config.type === 'once') {
			Ext.apply(result, {
				type: 'O',
				year: period.period_year,
				month: period.period_month,
				day: period.period_days[0].period_day
			});
		} else { // repeat
			if (period.period_weeks) { // weekly
				var tmp = [];
				Ext.each(period.period_weeks, function(week) {
					tmp.push(week.period_week);
				});
				Ext.apply(result, {
					type: 'W',
					week: tmp
				});

			} else if (period.period_days) { // monthly
				var tmp = [];
				Ext.each(period.period_days, function(day) {
					tmp.push(day.period_day);
				});
				Ext.apply(result, {
					type: 'M',
					day: tmp
				});
			} else { //daily
				Ext.apply(result, {
					type: 'D'
				});
			}
		}

		Ext.apply(result, {
			hour: period.period_hour,
			minute: period.period_minute
		});

		return result;
	},

	handleAction: function(id) {
		var fn = this;

		fn.btnAction.disable();

		if (!id) {
			var fn = this,
				sm = fn.gridMain.getSelectionModel();

			id = [];

			Ext.each(sm.getSelection(), function(selection) {
				id.push(selection.data.id);
			});
		}

		if (sm.getSelection()[0].data.status === fn.status.BACKING_UP) { // backing up
			fn.handleAbort(id.join(','));
		} else {
			fn.handleBackup(id.join(','));
		}
	},

	handleAbort: function(id) {
		var fn = this;

		fn.win.el.mask(_S('COMMON', 'APPLYING') + '...');

		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(fn.API.task, { act: 'abort' }),
			method: 'post',
			params: {
				id: id
			},
			success: function(json, opts) {
				fn.gridMain.store.load(function() {
					fn.win.el.unmask();
				});
			},
			failure: function(json, opts) {
				fn.win.el.unmask();

				AS.ARC.util.showMsgWindow({
					5006: _S('ERROR', 'UNABLE_OBTAIN_JOB_INFORMATION'),
					5008: _S('MESSAGE', 'HN00006'),
					5011: _S('HIDRIVE_BACKUP', 'UNABLE_ABORT_PROGRESS'),
					5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
					5028: _S('HIDRIVE_BACKUP', 'JOB_NOT_EXIST')
				}, json, fn.win);
			}
		});
	},

	handleBackup: function(id) {
		var fn = this;

		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(fn.API.task, { act: 'run' }),
			method: 'post',
			params: {
				id: id
			},
			success: function(json, opts) {
				fn.gridMain.store.load(function() {
					fn.btnAction.enable();
				});
			},
			failure: function(json, opts) {
				AS.ARC.util.showMsgWindow({
					5011: _S('HIDRIVE_BACKUP', 'UNABLE_EXE_EDIT'),
					5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
					5028: _S('HIDRIVE_BACKUP', 'JOB_NOT_EXIST'),
					5613: _S('HIDRIVE_BACKUP', 'UNABLE_EXE_PROGRESS')
				}, json, fn.win);
				fn.btnAction.enable();
			}
		});
	},

	getEditData: function(type) {
		var fn = this,
			sm = fn.gridMain.getSelectionModel();

		if (sm.getCount() !== 1) {
			return;
		}

		fn.btnEdit.disable();
		fn.btnInfo.disable();

		fn.win.el.mask(_S('COMMON', 'LOADING') + '...');

		var id = sm.getSelection()[0].data.id;

		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(fn.API.task, { act: 'detail' }),
			method: 'post',
			params: {
				id: id
			},
			success: function(json, opts) {
				Ext.apply(json, {
					id: id
				});

				fn.win.el.unmask();

				if (type === 'info') {
					fn.handleInfo(json);
				} else {
					fn.handleEdit(json);
				}

				fn.btnEdit.enable();
				fn.btnInfo.enable();
			},
			failure: function(json, opts) {
				fn.win.el.unmask();
				fn.btnEdit.enable();
				fn.btnInfo.enable();

				AS.ARC.util.showMsgWindow({
					5011: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
					5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
					5028: _S('ERROR', 'UNABLE_OBTAIN_JOB_INFORMATION')
				}, json, fn.win);
			}
		});
	},

	// This config is the information form the task
	handleEdit: function(config) {
		var fn = this,
			config = config || {},
			task = config.task;

		fn.originPath = [];

		Ext.each(task.paths.local, function(local) {
			fn.originPath.push(local.path);
		});

		var checkType = function() {
			var infoItem = panelOption.items,
				protocol = panelMode.items.get('protocol').getValue();

			infoItem.get('one').hide();

			if (protocol === 'RSYNC' || protocol === 'RSYNC+SSH') {
				//infoItem.get('exist').hide();
				infoItem.get('archive').show();
				infoItem.get('compression').show();
				// infoItem.get('metadata').show();
				infoItem.get('sparse').show();
				// infoItem.get('one').show();
				infoItem.get('deletion').show();
			} else {
				//infoItem.get('exist').show();
				infoItem.get('archive').hide();
				infoItem.get('compression').hide();
				// infoItem.get('metadata').hide();
				infoItem.get('sparse').hide();
				// infoItem.get('one').hide();
				infoItem.get('deletion').show();
			}
		};

		var panelMode = Ext.create('AS.ARC.panelTop10', {
			title: _S('COMMON', 'CONNECTION'),
			itemId: 'mode',
			border: false,
			defaults: {
				labelWidth: AS.ARC.config.layout.defaultLabelWidth,
				msgTarget: AS.ARC.config.msgTarget,
				checkChangeBuffer: AS.ARC.config.keyBuffer,
				blankText: _S('MESSAGE', 'HF00011')
			},
			items: [{
				xtype: 'displayfield',
				fieldLabel: _S('COMMON', 'PROTOCOL'),
				itemId: 'protocol',
				value: task.protocol
			}, {
				xtype: 'displayfield',
				itemId: 'mode',
				fieldLabel: _S('REMOTESYNC', 'TRANSFER_MODE'),
				value: (task.transfer_mode === 'to') ? _S('HIDRIVE_BACKUP', 'NAS_TO_HIDRIVE') : _S('HIDRIVE_BACKUP', 'HIDRIVE_TO_NAS')
			}, {
				xtype: 'displayfield',
				itemId: 'username',
				fieldLabel: _S('COMMON', 'USERNAME'),
				// allowBlank: false,
				value: task.username
				// listeners: {
				// change: function () {
				// if ((task.transfer_mode === 'from')) {
				// panelSource.resetSourceTree()
				// }
				// }
				// }
			}, {
				xtype: 'textfield',
				itemId: 'password',
				inputType: 'password',
				fieldLabel: _S('COMMON', 'PWD'),
				value: '111111',
				changeFlag: false,
				listeners: {
					change: function(f) {
						f.changeFlag = true;

						if ((task.transfer_mode === 'from')) {
							panelSource.resetSourceTree()
						}
					},
					focus: function(f) {
						if (!f.changeFlag) {
							f.setValue('');
						}
					},
					blur: function(f) {
						if (f.getValue() == '') {
							f.setValue('111111');
							f.changeFlag = false;
						}
					}
				}
			}, {
				xtype: 'button',
				itemId: 'try',
				text: _S('HIDRIVE_BACKUP', 'TEST_CONNECTION'),
				margin: '15 0 0 0',
				handler: function() {
					var cfg = {
						params: {}
					},
						pwdField = panelMode.down('[itemId=password]'),
						username = panelMode.down('[itemId=username]').getValue(),
						password = pwdField.getValue();

					if (pwdField.changeFlag) {
						Ext.apply(cfg.params, {
							username: username,
							password: password,
							protocol: task.protocol
						});
					} else {
						Ext.apply(cfg.params, {
							id: config.id
						});
					}

					Ext.apply(cfg, {
						win: win,
						cb: function() {
							var successWin = new AS.ARC.msgWindow({
								title: _S('COMMON', 'INFORMATION'),
								parentWin: win,
								width: 400,
								iconType: 'info',
								asItems: [{
									xtype: 'displayfield',
									value: _S('COMMON', 'SUCCESS')
								}],
								fbar: [{
									text: _S('COMMON', 'OK'),
									handler: function() {
										successWin.close();
									}
								}]
							}).show();
						}
					});

					fn.tryConnect(cfg);
				}
			}]
		});

		var panelSource = Ext.create('AS.ARC.panelTop10', {
			title: _S('REMOTESYNC', 'BACKUP_FOLDERS'),
			itemId: 'source',
			items: [{
				xtype: 'displayfield',
				value: _S('REMOTESYNC', 'SELECT_SOURCE_FOLDERS_BACKUP')
			}],
			resetSourceTree: function() {
				if (!panelSource.tree) { return; }

				panelSource.tree.destroy();
				panelSource.tree = null;
			},
			listeners: {
				afterrender: function(panel) {
					if (!panel.tree) {
						var extraParam = {};

						if (task.transfer_mode === 'from') {
							var pwdField = panelMode.items.get('password'),
								username = panelMode.items.get('username').getValue(),
								password = pwdField.getValue();

							if (pwdField.changeFlag) {
								Ext.apply(extraParam, {
									username: username,
									password: password,
									protocol: task.protocol
								});
							} else {
								Ext.apply(extraParam, {
									id: config.id
								});
							}
						} else {
							Ext.apply(extraParam, {
								id: config.id
							});
						}

						if (task.transfer_mode === 'to') {
							var tree = fn.getTreeNas('edit', 'source', win, fn.buttonApply);
						} else {

							var tree = Ext.widget('appHidriveTree', {
								animate: false,
								// height: 235,
								apiUrl: (task.transfer_mode === 'from') ? fn.API.remote : fn.API.local,
								extraParam: extraParam,
								listeners: {
									exception: function(proxy, response, operation, opts) {
										if (operation.error && operation.error.status === -1) { return; }

										var json = Ext.JSON.decode(response.responseText);
										if (json) { // not aborted
											AS.ARC.util.showMsgWindow({
												5001: _S('ERROR', 'PRIVATEKEY_ERROR'),
												5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
												5025: _S('ERROR', 'NO_OBTAIN_LIST_RIGHTS'),
												5028: _S('HIDRIVE_BACKUP', 'JOB_NOT_EXIST'),
												5039: _S('ERROR', 'CONNECTION_FIALED'),
												5801: _S('ERROR', 'CONNECTION_FIALED')
											}, json, win, 'warning', function() {
												operation.node.collapse();
												if (json.error_code == 5001) {
													win.items.getAt(0).setActiveTab(0);
												}
											});
										}
									}
								}
							});
						}

						panel.tree = panel.insert(1, tree);
					}
				}
			}
		});

		var sysPath = Ext.create('Ext.form.field.Text', {
			itemId: 'dest',
			allowBlank: false,
			blankText: _S('MESSAGE', 'HF00011'),
			readOnly: true,
			flex: true,
			value: (task.transfer_mode === 'to') ? task.paths.remote[0].path : task.paths.local[0].path,
			//disabled: true,
			editable: false
		});

		var sysBrowse = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'BROWSE'),
			margin: '0 0 0 5',
			//disabled: true,
			handler: function() {

				var fs = Ext.create('AS.ARC.treeSelector', {
					hideReadOnly: true,
					parentWin: win,
					objectPath: sysPath
				});
				fs.win.show();
			}
		});

		var pathContainer = Ext.create('Ext.container.Container', {
			itemId: 'pathContainer',
			layout: 'hbox',
			defaults: {
				blankText: _S('MESSAGE', 'HF00011')
			},
			items: [
				sysPath,
				sysBrowse
			]
		});


		var panelDest = Ext.create('AS.ARC.panelTop10', {
			title: _S('REMOTESYNC', 'DESTINATION_FOLDER'),
			itemId: 'destination',
			items: [{
				xtype: 'displayfield',
				value: _S('FTPBACKUP', 'SELECT_FOLDER_BACKUP')
			},
			{
				xtype: 'appHidrivePath',
				itemId: 'dest',
				getParentWin: function() {
					return win;
				}
			},
				pathContainer
			],
			listeners: {
				afterrender: function(panel) {
					var extraParam = {};

					if (task.transfer_mode === 'to') {
						var pwdField = panelMode.items.get('password'),
							username = panelMode.items.get('username').getValue(),
							password = pwdField.getValue();

						if (pwdField.changeFlag) {
							Ext.apply(extraParam, {
								username: username,
								password: password,
								protocol: panelMode.items.get('protocol').getValue()
							});
						} else {
							Ext.apply(extraParam, {
								id: config.id
							});
						}
					}

					if ((task.transfer_mode === 'to')) {

						panelDest.items.get('dest').show();
						panelDest.items.get('pathContainer').hide();

						panelDest.items.get('dest').setValue((task.transfer_mode === 'to') ? task.paths.remote[0].path : task.paths.local[0].path);
						panel.items.get('dest').apiUrl = (task.transfer_mode === 'to') ? fn.API.remote : fn.API.local;
						panel.items.get('dest').createAct = (task.transfer_mode === 'to') ? 'create' : 'createdir';
						panel.items.get('dest').createApiUrl = (task.transfer_mode === 'to') ? fn.API.remote : fn.API.fileExplorer;
						panel.items.get('dest').extraParam = extraParam;
						panelDest.items.get('pathContainer').items.get('dest').setValue('1');

					} else {

						panelDest.items.get('dest').hide();
						panelDest.items.get('pathContainer').show();
						panel.items.get('pathContainer').items.get('dest').apiUrl = (task.transfer_mode === 'to') ? fn.API.remote : fn.API.local;
						panel.items.get('pathContainer').items.get('dest').createAct = (task.transfer_mode === 'to') ? 'create' : 'createdir';
						panel.items.get('pathContainer').items.get('dest').createApiUrl = (task.transfer_mode === 'to') ? fn.API.remote : fn.API.fileExplorer;
						panel.items.get('pathContainer').items.get('dest').extraParam = extraParam;
						panelDest.items.get('dest').setValue('1');
					}
				}
			}
		});

		var panelSchedule = Ext.create('AS.ARC.panelTop10', {
			title: _S('EXT_BACKUP', 'SCHEDULE'),
			itemId: 'schedule',
			border: false,
			defaults: {
				labelWidth: AS.ARC.config.layout.defaultLabelWidth + (AS.ARC.config.layout.indentWidth * 1),
				msgTarget: AS.ARC.config.msgTarget,
				width: 425,
				blankText: _S('MESSAGE', 'HF00011')
			},
			items: fn.panelScheduleItems(true, task.mission_settings),
			listeners: {
				afterrender: function(panel) {
					var dateCmp = panel.items.get('date'),
						timeCmp = panel.items.get('time');

					dateCmp.setDateFormat(config.date_format);
					timeCmp.setTimeFormat(config.time_format);

					if (task.schedule) {
						panel.items.get('schedule').setValue(true);
						var schedule = fn.scheduleDecode(task.schedule);

						panel.items.get('frequency').setValue(schedule.type);

						switch (schedule.type) {
							case 'O':
								dateCmp.setValue(new Date(schedule.month + '/' + schedule.day + '/' + schedule.year));
								break;
							case 'M':
								panel.items.get('month-day').setValue(schedule.day);
								break;
							case 'W':
								panel.items.get('week-day').setValue(schedule.week);
								break;
							default:
							case 'D':
								break;
						}

						timeCmp.setTime({
							hour: schedule.hour,
							minute: schedule.minute
						});
					}
				}
			}
		});

		var panelOption = Ext.create('AS.ARC.panelTop10', {
			title: _S('HIDRIVE_BACKUP', 'OPTION'),
			itemId: 'option',
			border: false,
			defaults: {
				labelWidth: AS.ARC.config.layout.defaultLabelWidth,
				msgTarget: AS.ARC.config.msgTarget,
				checkChangeBuffer: AS.ARC.config.keyBuffer,
				blankText: _S('MESSAGE', 'HF00011')
			},
			items: fn.panelOptionItems(task),
			listeners: {
				afterrender: function(panel) {
					checkType();
					panel.addCls('hidrive-backup option');
				}
			}
		});

		fn.buttonApply = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'OK'),
			handler: function() {
				onBeforeApply();
			},
			scope: fn
		});

		var getSourcePath = function(oriSource) {
			var result = [];

			if (!panelSource.tree) {
				Ext.each(oriSource, function(oriPath) {
					result.push(oriPath.path);
				});
				return result;

			} else if (panelSource.tree.getHalfPathWithoutChild().length <= 0) {
				return panelSource.tree.getPath();
			}

			var paths = panelSource.tree.getHalfPathWithoutChild();

			Ext.each(oriSource, function(oriPath) {
				Ext.each(paths, function(path) {
					if (oriPath.path.indexOf(path) === 0) {
						result.push(oriPath.path);
						return false;
					}
				});
			});

			result = result.concat(panelSource.tree.getPath());
			return result;

		};

		var onBeforeApply = function() {
			var modeItems = panelMode.items,
				infoItems = panelOption.items,
				methodItems = panelSchedule.items,
				schedule = {};

			if (methodItems.get('schedule').getValue()) {
				timeCmp = methodItems.get('time');

				var cfg = {
					frequency: methodItems.get('frequency').getValue(),
					date: methodItems.get('date').getValue(),
					monthDay: methodItems.get('month-day').getValue(),
					weekDay: methodItems.get('week-day').getValue()
				};

				Ext.apply(schedule, fn.scheduleEncode(cfg));

				Ext.apply(schedule, timeCmp.getTime('hour'));
			}

			if (task.transfer_mode === 'to') {
				var sourcePath = fn.getSelection(fn.treeNas);
			} else {
				var sourcePath = getSourcePath(task.paths.remote);
			}

			var missionMode = methodItems.get('mission').missionMode;

			var params = {
				id: config.id,
				name: infoItems.get('name').getValue(),
				transfer_mode: task.transfer_mode,
				username: modeItems.get('username').getValue(),
				protocol: task.protocol,
				local: (task.transfer_mode === 'to') ? fn.getSelection(panelSource.items.get('tree-nas')) : panelDest.items.get('pathContainer').items.get('dest').getValue(),
				remote: (task.transfer_mode === 'to') ? panelDest.items.get('dest').getValue() : sourcePath.join('//'),
				deletion: !infoItems.get('deletion').getValue(),
				partial: infoItems.get('partial').getValue(),
				bandwidth: infoItems.get('bandwidth').items.get('enable').getValue() ? infoItems.get('bandwidth').items.get('value').getValue() * 1024 : 0,
				mission_mode: missionMode.getValue('enable'),
				retry_mode: (missionMode.getValue('mission') == 1) ? 'number_limit' : 'time_limit',
				retry_period: missionMode.getValue('period'),
				retry_interval: missionMode.getValue('interval'),
				retries: missionMode.getValue('attempt'),
				timeout: missionMode.getValue('timeout')
			};

			if (modeItems.get('password').changeFlag) {
				Ext.apply(params, {
					password: modeItems.get('password').getValue()
				});
			}

			Ext.apply(params, schedule);

			if (params.protocol === 'RSYNC' || params.protocol === 'RSYNC+SSH') {
				Ext.apply(params, {
					archive: infoItems.get('archive').getValue(),
					compression: infoItems.get('compression').getValue(),
					metadata: infoItems.get('metadata').getValue(),
					sparse: infoItems.get('sparse').getValue()
				});
			}

			// console.log(params);

			AS.ARC.ajax({
				url: AS.ARC.util.getApiUrlWithSid(fn.API.task, { act: 'update' }),
				method: 'post',
				params: params,
				win: win,
				success: function(json, opts) {
					fn.gridMain.store.load();
					win.close();
				},
				failure: function(json, opts) {
					win.el.unmask();

					AS.ARC.util.showMsgWindow({
						5011: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
						5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
						5008: _S('HIDRIVE_BACKUP', 'UNABLE_EDIT_EDIT'),
						5028: _S('HIDRIVE_BACKUP', 'JOB_NOT_EXIST'),
						5613: _S('HIDRIVE_BACKUP', 'UNABLE_EDIT_PROGRESS')
					}, json, win);
				}
			});
		};

		var tabPanel = Ext.create('AS.ARC.formTabPanel', {
			//buttonApply: fn.buttonApply,
			items: [
				panelMode,
				panelSource,
				panelDest,
				panelSchedule,
				panelOption
			],
			deferredRender: false
		});

		var win = Ext.create('AS.ARC.subWindow', {
			title: _S('COMMON', 'EDIT'),
			parentWin: fn.win,
			width: 600,
			height: 455,
			layout: 'fit',
			itemId: 'edit',
			items: [
				tabPanel
			],
			fbar: {
				items: [
					fn.buttonApply,
					{
						text: _S('COMMON', 'CANCEL'),
						scale: AS.ARC.config.defaultApplyButtonSize,
						handler: function() {
							win.close();
						},
						scope: this
					}]
			},
			listeners: {
				afterrender: function(me) {
					if (me.down('#tree')) {
						me.el.mask(_S('COMMON', 'LOADING'));
						me.down('#tree').parentWin = me;
					}
				}
			}
		}).show();
	},

	missionMode: function() {
		var enable = false,
			period = 60,
			attempt = 3,
			interval = 60,
			timeout = 60,
			mission = 1;

		this.setValue = function(value) {
			enable = value.enable;
			period = value.period;
			attempt = value.attempt;
			interval = value.interval;
			timeout = value.timeout;
			mission = value.mission;
		};

		this.getValue = function(tag) {
			if (tag) {
				if (tag === 'enable') {
					return enable;
				} else if (tag === 'period') {
					return period;
				} else if (tag === 'interval') {
					return interval;
				} else if (tag === 'timeout') {
					return timeout;
				} else if (tag === 'attempt') {
					return attempt;
				} else if (tag === 'mission') {
					return mission;
				}
			} else {
				return {
					enable: enable,
					period: period,
					attempt: attempt,
					interval: interval,
					timeout: timeout,
					mission: mission
				};
			}
		}
	},

	handleMissionMode: function(button, missionMode, parentWin) {
		var page = this;

		button.disable();

		var buttonApply = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'OK'),
			itemId: 'buttonApply',
			handler: function() {
				missionMode.setValue({
					enable: panel.items.get('enable').getValue(),
					mission: panel.items.get('attempt').items.get('enable').getValue() ? 1 : 2,
					interval: panel.items.get('interval').getValue(),
					timeout: panel.items.get('timeout').getValue(),
					attempt: panel.items.get('attempt').items.get('value').getValue(),
					period: panel.items.get('period').items.get('value').getValue()
				});
				win.close();
			}
		});

		var handleEnable = function(checkbox, checked) {
			if (checked) {
				panel.items.get('interval').enable();
				panel.items.get('timeout').enable();
				panel.items.get('attempt').items.get('enable').enable();
				panel.items.get('attempt').items.get('value').enable();
				panel.items.get('period').items.get('enable').enable();
				panel.items.get('period').items.get('value').enable();
				panel.items.get('note').enable();
			} else {
				panel.items.get('interval').disable();
				panel.items.get('timeout').disable();
				panel.items.get('attempt').items.get('enable').disable();
				panel.items.get('attempt').items.get('value').disable();
				panel.items.get('period').items.get('enable').disable();
				panel.items.get('period').items.get('value').disable();
				panel.items.get('note').disable();
			}
		};

		var panel = Ext.create('AS.ARC.panelTop20', {
			itemId: 'panel',
			border: false,
			defaults: {
				labelWidth: AS.ARC.config.layout.labelWidthLong + (AS.ARC.config.layout.indentWidth * 1),
				msgTarget: AS.ARC.config.msgTarget,
				allowBlank: false,
				blankText: _S('MESSAGE', 'HF00011')
			},
			items: [{
				xtype: 'checkbox',
				itemId: 'enable',
				boxLabel: _S('BACKUP_RESTORE', 'ENABLE_MISSION_MODE'),
				listeners: {
					change: handleEnable
				}
			}, {
				xtype: 'numberfield',
				itemId: 'timeout',
				disabled: true,
				fieldLabel: AS.layout.getIndent(1, _S('BACKUP_RESTORE', 'TIMEOUT')),
				value: 60,
				maxValue: 600,
				minValue: 60,
				nanText: _S('MESSAGE', 'HF00027'),
				maxText: Ext.String.format(_S('MESSAGE', 'HF00024'), 600),
				minText: Ext.String.format(_S('MESSAGE', 'HF00025'), 60)
			}, {
				xtype: 'numberfield',
				itemId: 'interval',
				disabled: true,
				fieldLabel: AS.layout.getIndent(1, _S('BACKUP_RESTORE', 'RECONNECTION_INTERVAL')),
				value: 60,
				maxValue: 300,
				minValue: 10,
				nanText: _S('MESSAGE', 'HF00027'),
				maxText: Ext.String.format(_S('MESSAGE', 'HF00024'), 300),
				minText: Ext.String.format(_S('MESSAGE', 'HF00025'), 10)
			}, {
				itemId: 'attempt',
				border: false,
				layout: 'hbox',
				style: AS.layout.getIndentCss(1),
				items: [{
					xtype: 'radio',
					name: 'mission',
					itemId: 'enable',
					disabled: true,
					width: AS.ARC.config.layout.labelWidthLong + 2,
					boxLabel: _S('BACKUP_RESTORE', 'RECONNECTION_ATTEMPTS') + _S('COMMON', 'COLON'),
					checked: true
				}, {
					xtype: 'numberfield',
					maxValue: 120,
					minValue: 1,
					nanText: _S('MESSAGE', 'HF00027'),
					maxText: _S('MESSAGE', 'HF00024'),
					minText: _S('MESSAGE', 'HF00025'),
					disabled: true,
					itemId: 'value',
					allowBlank: false,
					blankText: _S('MESSAGE', 'HF00011'),
					msgTarget: AS.ARC.config.msgTarget,
					value: 3
				}]
			}, {
				itemId: 'period',
				border: false,
				layout: 'hbox',
				style: AS.layout.getIndentCss(1),
				items: [{
					xtype: 'radio',
					itemId: 'enable',
					disabled: true,
					width: AS.ARC.config.layout.labelWidthLong + 2,
					name: 'mission',
					boxLabel: _S('BACKUP_RESTORE', 'TIME_PERIOD') + _S('COMMON', 'COLON'),
					listeners: {
						change: function(radio, checked) {
							if (checked) {
								panel.items.get('note').show();
							} else {
								panel.items.get('note').hide();
							}
						}
					}
				}, {
					xtype: 'numberfield',
					maxValue: 1440,
					minValue: 1,
					nanText: _S('MESSAGE', 'HF00027'),
					maxText: _S('MESSAGE', 'HF00024'),
					minText: _S('MESSAGE', 'HF00025'),
					disabled: true,
					itemId: 'value',
					allowBlank: false,
					blankText: _S('MESSAGE', 'HF00011'),
					msgTarget: AS.ARC.config.msgTarget,
					value: 60
				}]
			}, {
				xtype: 'displayfield',
				itemId: 'note',
				width: 500,
				value: '',
				hidden: true,
				disabled: true,
				style: AS.layout.getIndentCss(1),
				listeners: {
					afterrender: function(field) {
						field.setValue(_S('BACKUP_RESTORE', 'REMAIN_ACTIVE'));
					}
				}
			}]
		});

		var win = Ext.create('AS.ARC.subWindow', {
			title: _S('COMMON', 'MISSIONMODE'),
			parentWin: parentWin,
			width: 560,
			height: 380,
			layout: 'fit',
			items: [
				panel
			],
			fbar: [
				buttonApply, {
					text: _S('COMMON', 'CANCEL'),
					scale: AS.ARC.config.defaultApplyButtonSize,
					handler: function() {
						win.close();
					},
					scope: this
				}],
			listeners: {
				afterrender: function() {
					panel.items.get('enable').setValue(missionMode.getValue('enable'));
					panel.items.get('interval').setValue(missionMode.getValue('interval'));
					panel.items.get('timeout').setValue(missionMode.getValue('timeout'));
					panel.items.get('attempt').items.get('value').setValue(missionMode.getValue('attempt'));
					panel.items.get('period').items.get('value').setValue(missionMode.getValue('period'));
					if (missionMode.getValue('mission') === 1) {
						panel.items.get('attempt').items.get('enable').setValue(true);
					} else {
						panel.items.get('period').items.get('enable').setValue(true);
					}
				},
				beforeclose: function() {
					button.enable();
				}
			}
		}).show();
	},

	handleRemove: function() {
		var fn = this,
			sm = fn.gridMain.getSelectionModel();

		var id = [];

		Ext.each(sm.getSelection(), function(selection) {
			id.push(selection.data.id);
		});

		fn.btnRemove.disable();

		var win = Ext.create('AS.ARC.msgWindow', {
			title: _S('COMMON', 'CONFIRMATION'),
			parentWin: fn.win,
			width: 400,
			iconType: 'question',
			asItems: [{
				xtype: 'displayfield',
				value: _S('COMMON', 'REMOVE_SELECTED_ITEMS')
			}],
			fbar: [{
				text: _S('COMMON', 'OK'),
				handler: function() {
					fn.btnRemove.enable();

					AS.ARC.ajax({
						url: AS.ARC.util.getApiUrlWithSid(fn.API.task, { act: 'remove' }),
						method: 'post',
						params: {
							id: id.join(',')
						},
						success: function(json, opts) {
							fn.gridMain.store.load();
						},
						failure: function(json, opts) {
							AS.ARC.util.showMsgWindow({
								5008: _S('HIDRIVE_BACKUP', 'ENABLE_REMOVE_EDIT'),
								5011: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
								5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
								5028: _S('HIDRIVE_BACKUP', 'ENABLE_REMOVE_PROGRESS'),
								5613: _S('HIDRIVE_BACKUP', 'ENABLE_REMOVE_PROGRESS')
							}, json, fn.win);
						}
					});
					win.close();
				},
				scope: this
			}, {
				text: _S('COMMON', 'CANCEL'),
				handler: function() {
					fn.btnRemove.enable();

					win.close();
				},
				scope: this
			}]
		}).show();
	},

	handleInfo: function(config) {
		var fn = this,
			config = config || {},
			task = config.task;

		var getSourcePath = function() {
			var paths = [];
			var source = (task.transfer_mode === 'to') ? task.paths.local : task.paths.remote;

			Ext.each(source, function(s) {
				paths.push(s.path);
			});

			return paths;
		};

		var getDestPath = function() {
			var paths = [];
			var dest = (task.transfer_mode === 'to') ? task.paths.remote : task.paths.local;

			Ext.each(dest, function(s) {
				paths.push(s.path);
			});

			return paths;
		};

		var getTime = function() {
			var schedule = fn.scheduleDecode(task.schedule);

			Ext.apply(schedule, {
				dateFormat: config.date_format,
				timeFormat: config.time_format
			});

			if (!schedule.type) {
				var result = '--';
			} else {
				var result = '';
				switch (schedule.type) {
					case 'O':
						result += _S('COMMON', 'ONCE') + ', ';
						result += ' ' + fn.getDateTimeString(schedule);
						break;
					case 'D':
						result += _S('COMMON', 'DAILY') + ', ';
						result += ' ' + fn.getTimeString(schedule);
						break;
					case 'W':
						var w = [_S('COMMON', 'MON'), _S('COMMON', 'TUE'), _S('COMMON', 'WED'), _S('COMMON', 'THU'), _S('COMMON', 'FRI'), _S('COMMON', 'SAT'), _S('COMMON', 'SUN')];
						var tmp = [];

						schedule.week.sort().forEach(function(day) {
							tmp.push(w[parseInt(day, 10) - 1]);
						});

						result += _S('COMMON', 'WEEKLY') + ', ';
						result += ' ' + fn.getTimeString(schedule);
						result += ', ' + _S('COMMON', 'INTERVAL').toLowerCase() + ': ' + tmp.join(',');
						break;
					case 'M':
						result += _S('COMMON', 'MONTHLY') + ', ';
						result += ' ' + fn.getTimeString(schedule);
						result += ', ' + _S('COMMON', 'INTERVAL').toLowerCase() + ': ' + schedule.day.sort(function(a, b) { return a - b; }).join(',');
						break;
				}
			}

			return result;
		};

		var store = Ext.create('Ext.data.Store', {
			fields: [
				'name',
				'value'
			],
			data: [{
				name: _S('COMMON', 'NAME'),
				value: task.name
			}, {
				name: _S('HIDRIVE_BACKUP', 'TRANSFER_TYPE'),
				value: task.protocol
			}, {
				name: _S('REMOTESYNC', 'TRANSFER_MODE'),
				value: (task.transfer_mode === 'to') ? _S('HIDRIVE_BACKUP', 'NAS_TO_HIDRIVE') : _S('HIDRIVE_BACKUP', 'HIDRIVE_TO_NAS')
			}, {
				name: _S('COMMON', 'USERNAME'),
				value: task.username
			}, {
				name: _S('COMMON', 'TIME'),
				value: getTime()
			}, {
				name: _S('REMOTESYNC', 'BACKUP_FOLDERS'),
				value: getSourcePath()//.join(',<br>')
			}, {
				name: _S('REMOTESYNC', 'DESTINATION_FOLDER'),
				value: getDestPath()//.join(',<br>')
			}, {
				name: _S('UPLOAD', 'PROGRESS'),
				value: '--'
			}]
		});

		fn.gridInfo = Ext.create('Ext.grid.Panel', {
			border: false,
			//height: 245,
			store: store,
			columns: [{
				text: _S('COMMON', 'NAME'),
				flex: 2,
				sortable: false,
				dataIndex: 'name'
			}, {
				text: _S('COMMON', 'VALUE'),
				flex: 5,
				sortable: false,
				dataIndex: 'value',
				renderer: function(value, metadata) {
					if (Ext.isArray(value)) {
						var tmp = '';
						Ext.each(value, function(item, index) {
							tmp += AS.ARC.util.replaceHtmlSymbols(item);
							if (index + 1 !== value.length) {
								tmp += ',<br>';
							}
						});
						value = tmp;
					} else {
						value = AS.ARC.util.replaceHtmlSymbols(value);
					}
					metadata.tdAttr = 'data-qtip="' + AS.ARC.util.replaceHtmlSymbols(value) + '"';
					return value;
				}
			}],
			listeners: {
				afterrender: function(grid) {

				},
				resize: function(grid) {
					if (grid.getHeight() > 217) {
						grid.setHeight(217);
						setTimeout(function() {
							grid.doLayout();
						}, 50);
					}
				}
			}
		});

		var buttonApply = Ext.create('Ext.button.Button', {
			text: _S('COMMON', 'OK'),
			handler: function() {
				fn.infoWin.close();
			}
		});

		fn.progressTitle = Ext.create('Ext.form.field.Display', {
			fieldLabel: _S('COMMON', 'STATUS'),
			labelWidth: AS.ARC.config.layout.labelWidthMedium,
			width: 650,
			hidden: true,
			value: ''
		});

		fn.progressBar = Ext.create('Ext.ProgressBar', {
			value: 0,
			hidden: true
		});

		fn.infoWin = Ext.create('AS.ARC.subWindow', {
			title: _S('COMMON', 'INFORMATION'),
			parentWin: fn.win,
			width: 700,
			height: 395,
			layout: 'fit',
			taskId: config.id,
			fbar: [buttonApply],
			items: [{
				xtype: 'asPanelTop10',
				items: [fn.gridInfo, fn.progressTitle, fn.progressBar]
			}],
			listeners: {
				afterrender: function() {
					fn.getProgress(config.id);
				},
				close: function() {
					clearTimeout(fn.progressId);
				}
			}
		}).show();
	},

	getProgress: function(id) {
		var fn = this;

		clearTimeout(fn.progressId);

		AS.ARC.ajax({
			url: AS.ARC.util.getApiUrlWithSid(fn.API.task, { act: 'progress' }),
			method: 'post',
			params: {
				id: id
			},
			success: function(json, opts) {
				// if (json.status == 0 || json.status == 4) {
				// if (fn.progressTitle.isVisible()) {
				// fn.progressTitle.setValue(_S('STATUS', 'FINISHED'));
				// }
				// } else if (json.status == 1) {
				// fn.progressTitle.show();
				// fn.progressTitle.setValue(_S('EXT_BACKUP', 'CURRENT_FILE'));

				// } else {
				// fn.progressTitle.show();
				// fn.progressTitle.setValue(json.current_name + '(' + json.current_percentage + '%)' );
				// }

				var record = fn.gridInfo.store.getAt(7);

				if (json.status == 0 || json.status == 4) {
					if (record.data.value !== '--') {
						record.set('value', _S('STATUS', 'FINISHED'));
					}
				} else if (json.status == 1) {
					record.set('value', _S('UPLOAD', 'IN_PROGRESS'));
				} else {
					record.set('value', _S('EXT_BACKUP', 'CURRENT_FILE') + ' - ' + json.current_name);
				}

				record.commit();

				if (fn.infoWin && fn.infoWin.isVisible() && id === fn.infoWin.taskId) {
					fn.progressId = setTimeout(function() {
						fn.getProgress(id);
					}, 3000);
				}
			},
			failure: function(json, opts) {
				// AS.ARC.util.showMsgWindow({
				// 5011: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
				// 5018: _S('HIDRIVE_BACKUP', 'UNEXPECT_ERROR'),
				// 5008: _S('HIDRIVE_BACKUP', 'UNABLE_EDIT_EDIT'),
				// 5028: _S('HIDRIVE_BACKUP', 'JOB_NOT_EXIST'),
				// 5613: _S('HIDRIVE_BACKUP', 'UNABLE_EDIT_PROGRESS')
				// }, json, config.win);
			}
		});
	},

	getList: function() {
		var fn = this;

		fn.gridMain.store.load({
			callback: function() {
				if (fn.list.stop !== true) {
					fn.list.id = setTimeout(function() {
						fn.getList();
					}, fn.list.timeout);
				}
			}
		});
	},

	loadData: function() {
		var fn = this;

		fn.list.stop = false;
		fn.getList();
	}
});

/**
 * @class AS.ARC.apps.hidriveBackup.main
 * @extends AS.ARC._appBase
 * <p>Main app</p>
 */
Ext.define('AS.ARC.apps.hidriveBackup.main', {
	extend: 'AS.ARC._appBase',

	appTag: 'hidrive-backup',
	title: _S('HIDRIVE_BACKUP', 'HIDRIVE_BACKUP'),
	appMaxNum: 1,
	appOpenNum: 0,
	appIsReady: true,
	appWins: [],

	createWindow: function() {
		var desktop = this.core.getDesktop(),
			app = this;

		if ((this.appOpenNum === this.appMaxNum) || !this.appIsReady) {
			this.appWins[0].show();
			return;
		}

		// Set appIsReady to false
		this.appIsReady = false;

		var hidriveBackup = new AS.ARC.apps.hidriveBackup.core({
			app: this,
			desktop: desktop,
			id: this.id
		});

		hidriveBackup.win.on('render', function() {
			app.appOpenNum++;
			app.appIsReady = true;
		});

		hidriveBackup.win.on('beforeclose', function() {
			app.appOpenNum--;
			app.appIsReady = true;
			app.appWins.pop();
		});
		hidriveBackup.win.show();

		// Get all wins for handling
		this.appWins.push(hidriveBackup.win);
	}
});